@@ -14,7 +14,6 @@
 #endif
 
 #if	MSDOS
-extern int getcurdrv __P_((VOID_A));
 extern int setcurdrv __P_((int, int));
 #endif
 
@@ -27,23 +26,23 @@ extern int setcurdrv __P_((int, int));
 #  ifdef	SVR4
 #  include <sys/sysmacros.h>
 #   ifndef	major
-#   define	major(n)	((((unsigned long)(n)) >> 18) & 0x3fff)
+#   define	major(n)	((((u_long)(n)) >> 18) & 0x3fff)
 #   endif
 #   ifndef	minor
-#   define	minor(n)	(((unsigned long)(n)) & 0x3ffff)
+#   define	minor(n)	(((u_long)(n)) & 0x3ffff)
 #   endif
 #  else
 #   ifndef	major
-#   define	major(n)	((((unsigned)(n)) >> 8) & 0xff)
+#   define	major(n)	((((u_int)(n)) >> 8) & 0xff)
 #   endif
 #   ifndef	minor
-#   define	minor(n)	(((unsigned)(n)) & 0xff)
+#   define	minor(n)	(((u_int)(n)) & 0xff)
 #   endif
 #  endif
 # endif
 #endif
 
-#define	CL_NORM		0
+#define	CL_REG		0
 #define	CL_BACK		1
 #define	CL_DIR		2
 #define	CL_RONLY	3
@@ -53,11 +52,12 @@ extern int setcurdrv __P_((int, int));
 #define	CL_FIFO		7
 #define	CL_BLOCK	8
 #define	CL_CHAR		9
+#define	CL_EXE		10
 #define	ANSI_FG		8
 #define	ANSI_BG		9
 
 extern bindtable bindlist[];
-extern functable funclist[];
+extern CONST functable funclist[];
 #ifndef	_NOWRITEFS
 extern int writefs;
 #endif
@@ -94,19 +94,19 @@ static int NEAR putsize2 __P_((char *, namelist *, int));
 static int NEAR putfilename __P_((char *, namelist *, int));
 static VOID NEAR infobar __P_((VOID_A));
 static int NEAR calclocate __P_((int));
-static int NEAR calcfilepos __P_((namelist *, int, char *));
-static int NEAR listupmyself __P_((char *));
+static int NEAR calcfilepos __P_((namelist *, int, CONST char *));
+static int NEAR listupmyself __P_((CONST char *));
 #ifndef	_NOSPLITWIN
-static int NEAR listupwin __P_((char *));
+static int NEAR listupwin __P_((CONST char *));
 #endif
 static int NEAR searchmove __P_((int, char *));
 #ifndef	_NOPRECEDE
 static int readstatus __P_((VOID_A));
 #endif
-static int NEAR readfilelist __P_((reg_t *, char *));
+static int NEAR readfilelist __P_((CONST reg_t *, CONST char *));
 static VOID NEAR getfilelist __P_((VOID_A));
-static int NEAR browsedir __P_((char *, char *));
-static char *NEAR initcwd __P_((char *, char *, int));
+static int NEAR browsedir __P_((char *, CONST char *));
+static char *NEAR initcwd __P_((CONST char *, char *, int));
 
 int curcolumns = 0;
 int defcolumns = 0;
@@ -130,6 +130,9 @@ char fullpath[MAXPATHLEN] = "";
 char *macrolist[MAXMACROTABLE];
 int maxmacro = 0;
 int isearch = 0;
+#if	FD >= 2
+int helplayout = 0;
+#endif
 char *helpindex[MAXHELPINDEX] = {
 #ifdef	_NOTREE
 	"help",
@@ -148,14 +151,30 @@ char *helpindex[MAXHELPINDEX] = {
 #else
 	"Unpack",
 #endif
-};
-char typesymlist[] = "dbclsp";
-u_short typelist[] = {
-	S_IFDIR, S_IFBLK, S_IFCHR, S_IFLNK, S_IFSOCK, S_IFIFO
+#if	FD >= 2
+	"Attr", "Info", "Move", "rmDir", "mKdir",
+	"sHell",
+# ifdef	_NOWRITEFS
+	"",
+# else
+	"Write",
+# endif
+# ifdef	_NOARCHIVE
+	"",
+# else
+	"Backup",
+# endif
+	"View",
+# ifdef	_NOARCHIVE
+	"",
+# else
+	"Pack",
+# endif
+#endif	/* FD >= 2 */
 };
 #ifdef	HAVEFLAGS
-char fflagsymlist[] = "ANacuacu";
-u_long fflaglist[] = {
+CONST char fflagsymlist[] = "ANacuacu";
+CONST u_long fflaglist[] = {
 	SF_ARCHIVED, UF_NODUMP,
 	SF_APPEND, SF_IMMUTABLE, SF_NOUNLINK,
 	UF_APPEND, UF_IMMUTABLE, UF_NOUNLINK
@@ -171,11 +190,17 @@ char *precedepath = NULL;
 #endif
 #ifndef	_NOSPLITWIN
 int windows = 1;
+#endif
+#if	!defined (_NOSPLITWIN) || !defined (_NOPTY)
 int win = 0;
 #endif
 int calc_x = -1;
 int calc_y = -1;
 
+static CONST char typesymlist[] = "dbclsp";
+static CONST u_short typelist[] = {
+	S_IFDIR, S_IFBLK, S_IFCHR, S_IFLNK, S_IFSOCK, S_IFIFO
+};
 static CONST u_short modelist[] = {
 	S_IFDIR, S_IFLNK, S_IFSOCK, S_IFIFO, S_IFBLK, S_IFCHR
 };
@@ -189,7 +214,7 @@ static CONST u_char colorlist[] = {
 	CL_DIR, CL_LINK, CL_SOCK, CL_FIFO, CL_BLOCK, CL_CHAR
 };
 static CONST char defpalette[] = {
-	ANSI_FG,	/* CL_NORM */
+	ANSI_FG,	/* CL_REG */
 	ANSI_BG,	/* CL_BACK */
 	ANSI_CYAN,	/* CL_DIR */
 	ANSI_GREEN,	/* CL_RONLY */
@@ -199,6 +224,7 @@ static CONST char defpalette[] = {
 	ANSI_RED,	/* CL_FIFO */
 	ANSI_FG,	/* CL_BLOCK */
 	ANSI_FG,	/* CL_CHAR */
+	ANSI_FG,	/* CL_EXE */
 };
 #endif	/* !_NOCOLOR */
 #ifndef	_NOPRECEDE
@@ -220,8 +246,9 @@ namelist *namep;
 	for (i = 0; i < MAXMODELIST; i++)
 		if ((namep -> st_mode & S_IFMT) == modelist[i])
 			return(colorlist[i]);
+	if (isexec(namep)) return(CL_EXE);
 
-	return(CL_NORM);
+	return(CL_REG);
 }
 
 static int NEAR biascolor(color)
@@ -291,8 +318,20 @@ VOID helpbar(VOID_A)
 {
 	int i, j, col, gap, width, len, ofs, max, blk, rest;
 
-	max = MAXHELPINDEX;
-	blk = 5;
+#if	FD >= 2
+	if (helplayout) {
+		max = helplayout / 100;
+		blk = helplayout % 100;
+		if (max < 0) max = FUNCLAYOUT / 100;
+		else if (max > MAXHELPINDEX) max = MAXHELPINDEX;
+		if (blk <= 0 || blk > max) blk = max;
+	}
+	else
+#endif
+	{
+		max = FUNCLAYOUT / 100;
+		blk = FUNCLAYOUT % 100;
+	}
 
 	if (ishardomit()) {
 		col = n_column;
@@ -314,6 +353,7 @@ VOID helpbar(VOID_A)
 		col = n_column - 1;
 		gap = 3;
 	}
+	if (max > FUNCLAYOUT / 100) gap = 1;
 
 	rest = max - 1 + gap * ((max - 1) / blk);
 	width = (col - 4 - rest) / max;
@@ -344,7 +384,7 @@ VOID helpbar(VOID_A)
 
 static VOID NEAR statusbar(VOID_A)
 {
-	char *str[6];
+	CONST char *str[6];
 
 #ifndef	_NOTRADLAYOUT
 	if (istradlayout()) {
@@ -551,6 +591,18 @@ static VOID NEAR sizebar(VOID_A)
 	Xtflush();
 }
 
+u_int getfmode(c)
+int c;
+{
+	int i;
+
+	c = tolower2(c);
+	for (i = 0; typesymlist[i]; i++)
+		if (c == typesymlist[i]) return(typelist[i]);
+
+	return((u_int)-1);
+}
+
 int putmode(buf, mode, notype)
 char *buf;
 u_int mode;
@@ -1083,7 +1135,7 @@ int no, isstandout;
 static int NEAR calcfilepos(list, max, def)
 namelist *list;
 int max;
-char *def;
+CONST char *def;
 {
 	int i;
 
@@ -1099,10 +1151,10 @@ char *def;
 int listupfile(list, max, def, isstandout)
 namelist *list;
 int max;
-char *def;
+CONST char *def;
 int isstandout;
 {
-	char *cp;
+	CONST char *cp;
 	int i, n, pos;
 
 	for (i = 0; i < FILEPERROW; i++) {
@@ -1158,7 +1210,7 @@ int isstandout;
 }
 
 static int NEAR listupmyself(def)
-char *def;
+CONST char *def;
 {
 #ifndef	_NOTREE
 	if (treepath) rewritetree();
@@ -1180,7 +1232,7 @@ char *def;
 
 #ifndef	_NOSPLITWIN
 static int NEAR listupwin(def)
-char *def;
+CONST char *def;
 {
 	int i, x, y, n, dupwin;
 
@@ -1353,7 +1405,7 @@ static int NEAR searchmove(ch, buf)
 int ch;
 char *buf;
 {
-	char *str[4];
+	CONST char *str[4];
 	int i, n, s, pos, len;
 
 	if (isearch > 0) {
@@ -1467,8 +1519,8 @@ static int readstatus(VOID_A)
 #endif	/* !_NOPRECEDE */
 
 static int NEAR readfilelist(re, arcre)
-reg_t *re;
-char *arcre;
+CONST reg_t *re;
+CONST char *arcre;
 {
 #ifndef	_NOPRECEDE
 	char cwd[MAXPATHLEN];
@@ -1534,7 +1586,7 @@ static VOID NEAR getfilelist(VOID_A)
 	arcre = NULL;
 	if (findpattern) {
 #ifndef	_NOARCHIVE
-		if (*findpattern == '/') arcre = findpattern + 1;
+		if (*findpattern == '/') arcre = &(findpattern[1]);
 		else
 #endif
 		re = regexp_init(findpattern, -1);
@@ -1563,7 +1615,8 @@ static VOID NEAR getfilelist(VOID_A)
 
 	if (!maxfile) {
 		addlist();
-		filelist[0].name = NOFIL_K;
+		filelist[0].name = (char *)NOFIL_K;
+		filelist[0].tmpflags = 0;
 		filelist[0].st_nlink = -1;
 	}
 
@@ -1574,12 +1627,17 @@ static VOID NEAR getfilelist(VOID_A)
 }
 
 static int NEAR browsedir(file, def)
-char *file, *def;
+char *file;
+CONST char *def;
 {
+#ifndef	_NOARCHIVE
+	CONST char *tmp;
+#endif
 #ifndef	_NOPRECEDE
 	int dupsorton;
 #endif
-	char *cp, buf[MAXNAMLEN + 1];
+	CONST char *cp;
+	char buf[MAXNAMLEN + 1];
 	int ch, i, no, old, funcstat;
 
 #ifndef	_NOPRECEDE
@@ -1599,7 +1657,9 @@ char *file, *def;
 	waitmes();
 
 #ifndef	_NOARCHIVE
-	if (archivefile) def = (*file) ? file : nullstr;
+	if (!archivefile) /*EMPTY*/;
+	else if (*file) def = file;
+	else def = nullstr;
 #endif
 	getfilelist();
 
@@ -1766,8 +1826,6 @@ char *file, *def;
 			strcpy(file, filelist[filepos].name);
 		}
 		else if (no == FNC_CHDIR) {
-			char *tmp;
-
 			tmp = (filepos >= 0) ? filelist[filepos].name : NULL;
 			if (!(cp = archchdir(tmp))) {
 				if (!tmp) tmp = parentpath;
@@ -1812,7 +1870,8 @@ char *file, *def;
 }
 
 static char *NEAR initcwd(path, buf, evaled)
-char *path, *buf;
+CONST char *path;
+char *buf;
 int evaled;
 {
 	char *cp, *file;
@@ -1866,9 +1925,12 @@ int evaled;
 }
 
 VOID main_fd(pathlist, evaled)
-char **pathlist;
+char *CONST *pathlist;
 int evaled;
 {
+#ifdef	_USEDOSEMU
+	char buf[MAXPATHLEN];
+#endif
 	char *def, *cwd, file[MAXNAMLEN + 1], prev[MAXNAMLEN + 1];
 	int i, argc, ischgdir;
 
@@ -1960,17 +2022,12 @@ int evaled;
 			else copycurpath(file);
 		}
 
-		if (isdotdir(file) != 2) {
-#ifdef	_USEDOSEMU
-			char buf[MAXPATHLEN];
-#endif
-
-			if (chdir3(nodospath(buf, file), 1) < 0) {
-				hideclock = 2;
-				warning(-1, file);
-				strcpy(prev, file);
-				def = prev;
-			}
+		if (isdotdir(file) != 2
+		&& chdir3(nodospath(buf, file), 1) < 0) {
+			hideclock = 2;
+			warning(-1, file);
+			strcpy(prev, file);
+			def = prev;
 		}
 
 #ifdef	_NOARCHIVE
