@@ -20,8 +20,14 @@
 #include "termemu.h"
 #endif
 
+#ifndef	_NOIME
+#include "roman.h"
+#endif
+
 #define	MD5_BUFSIZ	(128 / 32)
 #define	MD5_BLOCKS	16
+#define	STRHDD		"HDD"
+#define	STRHDD98	"98"
 
 #ifndef	_NOARCHIVE
 extern launchtable launchlist[];
@@ -35,8 +41,8 @@ extern char **browsevar;
 extern char *macrolist[];
 extern int maxmacro;
 extern bindtable bindlist[];
-extern strtable keyidentlist[];
-extern functable funclist[];
+extern CONST strtable keyidentlist[];
+extern CONST functable funclist[];
 extern char **history[];
 extern short histsize[];
 extern short histno[];
@@ -48,7 +54,7 @@ extern int fd_restricted;
 #if	FD >= 2
 extern char *progpath;
 extern char *progname;
-extern char *promptstr;
+extern CONST char *promptstr;
 extern int lcmdline;
 #endif
 extern int internal_status;
@@ -63,77 +69,87 @@ extern int win_y;
 extern int hideclock;
 #endif
 extern int inruncom;
-#ifndef	_NOKANJICONV
+#ifdef	_USEUNICODE
 extern int unicodebuffer;
 #endif
 #ifndef	_NOPTY
 extern int parentfd;
 #endif
+#ifndef	_NOIME
+extern romantable *romanlist;
+extern int maxromanlist;
+#endif
 
-static VOID NEAR builtinerror __P_((char *[], char *, int));
+static VOID NEAR builtinerror __P_((char *CONST [], CONST char *, int));
 #ifdef	_NOORIGSHELL
 static VOID NEAR hitkey __P_((int));
 #else
 #define	hitkey(n)
 #endif
-static VOID NEAR fputsmeta __P_((char *, FILE *));
-#if	!defined (_NOARCHIVE) && !defined(_NOBROWSE)
-static char **NEAR file2argv __P_((FILE *, char *, int));
+static VOID NEAR fputsmeta __P_((CONST char *, FILE *));
+#if	(!defined (_NOARCHIVE) && !defined (_NOBROWSE)) || !defined (_NOIME)
+static char **NEAR file2argv __P_((FILE *, CONST char *, int));
 #endif
 #ifndef	_NOARCHIVE
 # if	FD >= 2
-static int NEAR getlaunchopt __P_((int, char *[], char *, launchtable *));
+static int NEAR getlaunchopt __P_((int, char *CONST [],
+		CONST char *, launchtable *));
 # endif
-static int NEAR setlaunch __P_((int, char *[]));
-static int NEAR setarch __P_((int, char *[]));
-static int NEAR printlaunch __P_((int, char *[]));
-static int NEAR printarch __P_((int, char *[]));
+static int NEAR setlaunch __P_((int, char *CONST []));
+static int NEAR setarch __P_((int, char *CONST []));
+static int NEAR printlaunch __P_((int, char *CONST []));
+static int NEAR printarch __P_((int, char *CONST []));
 # ifndef	_NOBROWSE
-static char **NEAR readargv __P_((char **, char **));
-static int NEAR custbrowse __P_((int, char *[]));
+static char **NEAR readargv __P_((char *CONST *, char **));
+static int NEAR custbrowse __P_((int, char *CONST []));
 # endif
 #endif
 static int NEAR getmacro __P_((char *));
-static int NEAR setkeybind __P_((int, char *[]));
-static int NEAR printbind __P_((int, char *[]));
+static int NEAR setkeybind __P_((int, char *CONST []));
+static int NEAR printbind __P_((int, char *CONST []));
 #ifdef	_USEDOSEMU
-static int NEAR _setdrive __P_((int, char *[], int));
-static int NEAR setdrive __P_((int, char *[]));
-static int NEAR unsetdrive __P_((int, char *[]));
-static int NEAR printdrive __P_((int, char *[]));
+static int NEAR _setdrive __P_((int, char *CONST [], int));
+static int NEAR setdrive __P_((int, char *CONST []));
+static int NEAR unsetdrive __P_((int, char *CONST []));
+static int NEAR printdrive __P_((int, char *CONST []));
 #endif
 #ifndef	_NOKEYMAP
-static int NEAR setkeymap __P_((int, char *[]));
-static int NEAR keytest __P_((int, char *[]));
+static int NEAR setkeymap __P_((int, char *CONST []));
+static int NEAR keytest __P_((int, char *CONST []));
 #endif
-static int NEAR printhist __P_((int, char *[]));
+static int NEAR printhist __P_((int, char *CONST []));
 #if	FD >= 2
 # ifndef	NOPOSIXUTIL
-static int NEAR fixcommand __P_((int, char *[]));
+static int NEAR fixcommand __P_((int, char *CONST []));
 # endif
 static VOID NEAR voidmd5 __P_((u_long, u_long, u_long, u_long));
 static VOID NEAR calcmd5 __P_((u_long [MD5_BUFSIZ], u_long [MD5_BLOCKS]));
-static int NEAR printmd5 __P_((char *, FILE *));
-static int NEAR md5sum __P_((int, char *[]));
-static int NEAR evalmacro __P_((int, char *[]));
+static int NEAR printmd5 __P_((CONST char *, FILE *));
+static int NEAR md5sum __P_((int, char *CONST []));
+static int NEAR evalmacro __P_((int, char *CONST []));
 # ifndef	_NOKANJICONV
-static int NEAR kconv __P_((int, char *[]));
+static int NEAR kconv __P_((int, char *CONST []));
 # endif
-static int NEAR getinputstr __P_((int, char *[]));
-static int NEAR getyesno __P_((int, char *[]));
+static int NEAR getinputstr __P_((int, char *CONST []));
+static int NEAR getyesno __P_((int, char *CONST []));
 #endif	/* FD >= 2 */
 #if	!MSDOS && (FD >= 2)
-static int NEAR savetty __P_((int, char *[]));
+static int NEAR savetty __P_((int, char *CONST []));
+#endif
+#ifndef	_NOIME
+static int NEAR setroman __P_((int, char *CONST []));
+static VOID NEAR disproman __P_((char *, int, FILE *));
+static int NEAR printroman __P_((int, char *CONST []));
 #endif
 #ifdef	_NOORIGSHELL
-static int NEAR printenv __P_((int, char *[]));
-static int NEAR setalias __P_((int, char *[]));
-static int NEAR unalias __P_((int, char *[]));
-static int NEAR checkuserfunc __P_((int, char *[]));
-static int NEAR setuserfunc __P_((int, char *[]));
-static int NEAR exportenv __P_((int, char *[]));
-static int NEAR dochdir __P_((int, char *[]));
-static int NEAR loadsource __P_((int, char *[]));
+static int NEAR printenv __P_((int, char *CONST []));
+static int NEAR setalias __P_((int, char *CONST []));
+static int NEAR unalias __P_((int, char *CONST []));
+static int NEAR checkuserfunc __P_((int, char *CONST []));
+static int NEAR setuserfunc __P_((int, char *CONST []));
+static int NEAR exportenv __P_((int, char *CONST []));
+static int NEAR dochdir __P_((int, char *CONST []));
+static int NEAR loadsource __P_((int, char *CONST []));
 #endif	/* _NOORIGSHELL */
 
 #if	FD >= 2
@@ -148,7 +164,7 @@ static int NEAR loadsource __P_((int, char *[]));
 #define	FUNCNAME	1
 #endif
 
-static CONST char *builtinerrstr[] = {
+static CONST char *CONST builtinerrstr[] = {
 	NULL,
 #define	ER_FEWMANYARG	1
 	"Too few or many arguments",
@@ -216,6 +232,10 @@ static CONST builtintable builtinlist[] = {
 #if	!MSDOS && (FD >= 2)
 	{savetty,	BL_SAVETTY},
 #endif
+#ifndef	_NOIME
+	{setroman,	BL_SETROMAN},
+	{printroman,	BL_PRINTROMAN},
+#endif
 #ifdef	_NOORIGSHELL
 # if	FD >= 2
 	{printenv,	BL_SET},
@@ -242,7 +262,8 @@ static CONST builtintable builtinlist[] = {
 
 
 static VOID NEAR builtinerror(argv, s, n)
-char *argv[], *s;
+char *CONST argv[];
+CONST char *s;
 int n;
 {
 	int duperrno;
@@ -292,7 +313,7 @@ int init;
 #endif	/* _NOORIGSHELL */
 
 static VOID NEAR fputsmeta(arg, fp)
-char *arg;
+CONST char *arg;
 FILE *fp;
 {
 	char *cp;
@@ -305,10 +326,10 @@ FILE *fp;
 	}
 }
 
-#if	!defined (_NOARCHIVE) && !defined(_NOBROWSE)
+#if	(!defined (_NOARCHIVE) && !defined (_NOBROWSE)) || !defined (_NOIME)
 static char **NEAR file2argv(fp, s, whole)
 FILE *fp;
-char *s;
+CONST char *s;
 int whole;
 {
 	char *cp, *line, **argv;
@@ -384,16 +405,17 @@ int whole;
 
 	return(argv);
 }
-#endif	/* !_NOARCHIVE && !_NOBROWSE */
+#endif	/* (!_NOARCHIVE && !_NOBROWSE) || !_NOIME */
 
 #ifndef	_NOARCHIVE
 # if	FD >= 2
 static int NEAR getlaunchopt(n, argv, opts, lp)
 int n;
-char *argv[], *opts;
+char *CONST argv[];
+CONST char *opts;
 launchtable *lp;
 {
-	char *cp;
+	CONST char *cp;
 	int i, c;
 
 	if (argv[n][0] != '-') return(0);
@@ -495,7 +517,7 @@ launchtable *lp;
 
 int parselaunch(argc, argv, lp)
 int argc;
-char *argv[];
+char *CONST argv[];
 launchtable *lp;
 {
 # if	FD >= 2
@@ -688,7 +710,7 @@ int n;
 
 static int NEAR setlaunch(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	launchtable launch;
 	int n;
@@ -755,7 +777,7 @@ archivetable *ap;
 
 int parsearch(argc, argv, ap)
 int argc;
-char *argv[];
+char *CONST argv[];
 archivetable *ap;
 {
 	int n;
@@ -811,7 +833,7 @@ int n;
 
 static int NEAR setarch(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	archivetable arch;
 	int n;
@@ -939,7 +961,7 @@ FILE *fp;
 
 static int NEAR printlaunch(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *ext;
 	int i, n;
@@ -995,7 +1017,7 @@ FILE *fp;
 
 static int NEAR printarch(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *ext;
 	int i, n;
@@ -1030,10 +1052,11 @@ char *argv[];
 
 # ifndef	_NOBROWSE
 static char **NEAR readargv(sargv, dargv)
-char **sargv, **dargv;
+char *CONST *sargv, **dargv;
 {
 	FILE *fp;
-	char *cp, **argv;
+	CONST char *cp;
+	char **argv;
 	int n, dargc;
 
 	dargc = countvar(dargv);
@@ -1093,7 +1116,7 @@ launchtable *list;
 
 static int NEAR custbrowse(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	launchtable *list;
 	char **argv2;
@@ -1225,7 +1248,7 @@ bindtable *list, *bindp;
 
 int parsekeybind(argc, argv, bindp)
 int argc;
-char *argv[];
+char *CONST argv[];
 bindtable *bindp;
 {
 	int i, n;
@@ -1356,7 +1379,7 @@ int n;
 
 static int NEAR setkeybind(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	bindtable bind;
 	char *cp, *func1, *func2;
@@ -1443,7 +1466,7 @@ FILE *fp;
 
 static int NEAR printbind(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	int i, c;
 
@@ -1630,7 +1653,7 @@ devinfo *devp;
 
 int parsesetdrv(argc, argv, devp)
 int argc;
-char *argv[];
+char *CONST argv[];
 devinfo *devp;
 {
 	int head, sect, cyl;
@@ -1657,7 +1680,7 @@ devinfo *devp;
 	}
 
 # ifdef	HDDMOUNT
-	if (!strncmp(argv[3], "HDD", strsize("HDD"))) {
+	if (!strncmp(argv[3], STRHDD, strsize(STRHDD))) {
 		if (argc > 5) {
 			devp -> drive = ER_FEWMANYARG;
 			devp -> head = 0;
@@ -1665,7 +1688,7 @@ devinfo *devp;
 		}
 		cyl = 0;
 		if (!argv[3][3]) sect = 0;
-		else if (!strcmp(&(argv[3][3]), "98")) sect = 98;
+		else if (!strcmp(&(argv[3][3]), STRHDD98)) sect = 98;
 		else {
 			devp -> drive = ER_SYNTAXERR;
 			devp -> head = 3;
@@ -1767,7 +1790,7 @@ devinfo *devp;
 
 static int NEAR _setdrive(argc, argv, isset)
 int argc;
-char *argv[];
+char *CONST argv[];
 int isset;
 {
 	devinfo dev;
@@ -1843,14 +1866,14 @@ int isset;
 
 static int NEAR setdrive(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	return(_setdrive(argc, argv, 1));
 }
 
 static int NEAR unsetdrive(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	return(_setdrive(argc, argv, 0));
 }
@@ -1868,8 +1891,8 @@ FILE *fp;
 	fputc('\t', fp);
 # ifdef	HDDMOUNT
 	if (!fdlist[n].cyl) {
-		fputs("HDD", fp);
-		if (isupper2(fdlist[n].head)) fputs("98", fp);
+		fputs(STRHDD, fp);
+		if (isupper2(fdlist[n].head)) fputs(STRHDD98, fp);
 		if (verbose) fprintf2(fp, " #offset=%'Ld",
 			fdlist[n].offset / fdlist[n].sect);
 	}
@@ -1882,7 +1905,7 @@ FILE *fp;
 
 static int NEAR printdrive(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	int i, n;
 
@@ -1934,7 +1957,7 @@ FILE *fp;
 
 int parsekeymap(argc, argv, kp)
 int argc;
-char *argv[];
+char *CONST argv[];
 keyseq_t *kp;
 {
 	kp -> code = (short)-1;
@@ -1973,7 +1996,7 @@ keyseq_t *kp;
 
 static int NEAR setkeymap(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	keyseq_t key;
 	int i;
@@ -2027,7 +2050,7 @@ char *argv[];
 
 static int NEAR keytest(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp, buf[2];
 	int i, n, ch, next;
@@ -2083,7 +2106,7 @@ char *argv[];
 
 static int NEAR printhist(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	int i, n, max, size;
 
@@ -2120,7 +2143,7 @@ char *argv[];
 # ifndef	NOPOSIXUTIL
 static int NEAR fixcommand(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	FILE *fp;
 	char *cp, *tmp, *editor, path[MAXPATHLEN];
@@ -2501,7 +2524,7 @@ u_long sum[MD5_BUFSIZ], x[MD5_BLOCKS];
 }
 
 static int NEAR printmd5(path, fp)
-char *path;
+CONST char *path;
 FILE *fp;
 {
 	FILE *fpin;
@@ -2517,7 +2540,7 @@ FILE *fp;
 	sum[3] = (u_long)0x10325476;
 
 	n = b = 0;
-	memset(x, 0, sizeof(x));
+	memset((char *)x, 0, sizeof(x));
 	while ((c = Xfgetc(fpin)) != EOF) {
 		if (cl <= (u_long)0xffffffff - (u_long)BITSPERBYTE)
 			cl += (u_long)BITSPERBYTE;
@@ -2533,7 +2556,7 @@ FILE *fp;
 			if (++n >= MD5_BLOCKS) {
 				n = 0;
 				calcmd5(sum, x);
-				memset(x, 0, sizeof(x));
+				memset((char *)x, 0, sizeof(x));
 			}
 		}
 	}
@@ -2542,7 +2565,7 @@ FILE *fp;
 	x[n] |= 1 << (b + BITSPERBYTE - 1);
 	if (n >= 14) {
 		calcmd5(sum, x);
-		memset(x, 0, sizeof(x));
+		memset((char *)x, 0, sizeof(x));
 	}
 	x[14] = cl;
 	x[15] = ch;
@@ -2560,7 +2583,7 @@ FILE *fp;
 
 static int NEAR md5sum(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char path[MAXPATHLEN];
 	int i, ret;
@@ -2594,7 +2617,7 @@ char *argv[];
 
 static int NEAR evalmacro(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp;
 	int ret;
@@ -2610,7 +2633,7 @@ char *argv[];
 # ifndef	_NOKANJICONV
 static int NEAR kconv(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	FILE *fpin, *fpout;
 	char *cp, *tmp;
@@ -2671,8 +2694,9 @@ char *argv[];
 		return(-1);
 	}
 
-	if (in == M_UTF8 || out == M_UTF8) readunitable(1);
-	else if (in == UTF8 || out == UTF8) readunitable(0);
+#  ifdef	_USEUNICODE
+	if ((i = (in > out) ? in : out) >= UTF8) readunitable(i - UTF8);
+#  endif
 	while ((cp = fgets2(fpin, 0))) {
 		if (in != DEFCODE) {
 			tmp = newkanjiconv(cp, in, DEFCODE, L_OUTPUT);
@@ -2686,7 +2710,9 @@ char *argv[];
 		free(tmp);
 	}
 
+#  ifdef	_USEUNICODE
 	if (!unicodebuffer) discardunitable();
+#  endif
 	if (fpin != stdin) Xfclose(fpin);
 	else clearerr(fpin);
 	if (fpout != stdout) Xfclose(fpout);
@@ -2697,9 +2723,10 @@ char *argv[];
 
 static int NEAR getinputstr(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
-	char *s, *duppromptstr;
+	CONST char *duppromptstr;
+	char *s;
 	int wastty;
 
 	if (dumbterm > 1) {
@@ -2708,7 +2735,8 @@ char *argv[];
 	}
 	if (!(wastty = isttyiomode)) ttyiomode(1);
 	duppromptstr = promptstr;
-	promptstr = (argc >= 2) ? argv[1] : nullstr;
+	if (argc >= 2) promptstr = argv[1];
+	else promptstr = nullstr;
 	s = inputstr(NULL, 0, -1, NULL, -1);
 	promptstr = duppromptstr;
 	if (!wastty) stdiomode();
@@ -2723,9 +2751,9 @@ char *argv[];
 
 static int NEAR getyesno(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
-	char *s;
+	CONST char *s;
 	int ret, wastty;
 
 	if (dumbterm > 1) {
@@ -2762,7 +2790,7 @@ int reset;
 # if	FD >= 2
 static int NEAR savetty(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	int n;
 
@@ -2776,10 +2804,153 @@ char *argv[];
 # endif	/* FD >= 2 */
 #endif	/* !MSDOS */
 
+#ifndef	_NOIME
+static int NEAR setroman(argc, argv)
+int argc;
+char *CONST argv[];
+{
+	FILE *fp;
+	char *file, **args;
+	int i, n, skip, clean;
+
+	file = NULL;
+	skip = clean = 0;
+	for (n = 1; n < argc && argv[n][0] == '-'; n++) {
+		skip = 0;
+		for (i = 1; argv[n][i]; i++) {
+			skip = 0;
+			switch (argv[n][i]) {
+				case 'c':
+					clean = 2;
+					break;
+				case 'r':
+					clean = 1;
+					break;
+				case 'f':
+					if (argv[n][i + 1]) {
+						file = &(argv[n][i + 1]);
+						skip = 1;
+					}
+					else if (n + 1 < argc) {
+						file = argv[++n];
+						skip = 1;
+					}
+					else skip = -1;
+					break;
+				default:
+					skip = -1;
+					break;
+			}
+			if (skip) {
+				skip--;
+				break;
+			}
+		}
+		if (skip) {
+			skip--;
+			break;
+		}
+	}
+	if (skip || (!file && !clean && n >= argc)) {
+		fprintf2(stderr,
+			"%k: usage: %k [-c] [-r] [-f file] [roman [kanji]]]",
+			argv[0], argv[0]);
+		fputnl(stderr);
+		return(-1);
+	}
+
+	if (clean) {
+		freeroman(clean - 1);
+# ifndef	 _NOPTY
+		sendparent(TE_FREEROMAN, clean - 1);
+# endif
+	}
+
+	if (!file) fp = NULL;
+	else if (!(fp = Xfopen(file, "r"))) {
+		builtinerror(argv, file, -1);
+		return(-1);
+	}
+	else for (;;) {
+		args = file2argv(fp, argv[0], 0);
+		i = 1;
+		if (args[i] && !strcommcmp(args[i], BL_SETROMAN)) i++;
+
+		if (!args[i]) /*EMPTY*/;
+		else if (addroman(args[i], args[i + 1]) < 0)
+			builtinerror(argv, args[i], ER_SYNTAXERR);
+# ifndef	 _NOPTY
+		else sendparent(TE_ADDROMAN, args[i], args[i + 1]);
+# endif
+		freevar(args);
+
+		if (Xfeof(fp)) break;
+	}
+	if (fp) Xfclose(fp);
+
+	if (n >= argc) /*EMPTY*/;
+	else if (addroman(argv[n], argv[n + 1]) < 0) {
+		builtinerror(argv, argv[n], ER_SYNTAXERR);
+		return(-1);
+	}
+# ifndef	 _NOPTY
+	else sendparent(TE_ADDROMAN, argv[n], argv[n + 1]);
+# endif
+
+	return(0);
+}
+
+static VOID NEAR disproman(s, n, fp)
+char *s;
+int n;
+FILE *fp;
+{
+	char buf[2 + 1];
+	int i;
+
+	if (s && *s) fprintf2(fp, "%s ", s);
+	fprintf2(fp, "%s \"", romanlist[n].str);
+	for (i = 0; i < R_MAXKANA; i++) {
+		if (!romanlist[n].code[i]) break;
+		VOID_C jis2str(buf, romanlist[n].code[i]);
+		fprintf2(fp, "%k", buf);
+	}
+	fputc('"', fp);
+	fputnl(fp);
+}
+
+static int NEAR printroman(argc, argv)
+int argc;
+char *CONST argv[];
+{
+	int i, n, ret;
+
+	initroman();
+	ret = -1;
+
+	if (argc < 2) {
+		for (i = 0; i < maxromanlist; i++)
+			disproman(BL_SETROMAN, i, stdout);
+		ret = 0;
+	}
+	else for (n = 1; n < argc; n++) {
+		if (!*(argv[n])
+		|| (i = searchroman(argv[n], strlen(argv[n]))) < 0) {
+			builtinerror(argv, argv[n], ER_NOENTRY);
+			continue;
+		}
+		disproman(BL_PRINTROMAN, i, stdout);
+		ret = 0;
+	}
+
+	return(ret);
+}
+#endif	/* !_NOIME */
+
 #ifdef	_NOORIGSHELL
 static int NEAR printenv(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 # if	FD < 2
 	int len;
@@ -2832,7 +3003,7 @@ char *argv[];
 }
 
 int searchalias(ident, len)
-char *ident;
+CONST char *ident;
 int len;
 {
 	int i;
@@ -2875,7 +3046,7 @@ char *ident, *comm;
 }
 
 int deletealias(ident)
-char *ident;
+CONST char *ident;
 {
 	reg_t *re;
 	int i, n;
@@ -2913,7 +3084,7 @@ FILE *fp;
 
 static int NEAR setalias(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp;
 	int i, len;
@@ -2982,7 +3153,7 @@ char *argv[];
 /*ARGSUSED*/
 static int NEAR unalias(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	if (argc != 2) {
 		builtinerror(argv, NULL, ER_FEWMANYARG);
@@ -2997,7 +3168,7 @@ char *argv[];
 }
 
 int searchfunction(ident)
-char *ident;
+CONST char *ident;
 {
 	int i;
 
@@ -3036,7 +3207,7 @@ char *ident, **comm;
 }
 
 int deletefunction(ident)
-char *ident;
+CONST char *ident;
 {
 	int i;
 
@@ -3074,7 +3245,7 @@ FILE *fp;
 
 static int NEAR checkuserfunc(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp;
 	int i, n;
@@ -3105,7 +3276,7 @@ char *argv[];
 
 static int NEAR setuserfunc(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp, *tmp, *line, **var, *list[MAXFUNCLINES + 1];
 	int i, j, n;
@@ -3187,7 +3358,7 @@ char *argv[];
 
 static int NEAR exportenv(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	char *cp;
 	int i;
@@ -3208,14 +3379,13 @@ char *argv[];
 	setenv2(argv[1], cp, 1);
 	if (cp) free(cp);
 	adjustpath();
-	evalenv();
 
 	return(0);
 }
 
 static int NEAR dochdir(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	if (argc != 2) {
 		builtinerror(argv, NULL, ER_FEWMANYARG);
@@ -3231,7 +3401,7 @@ char *argv[];
 
 static int NEAR loadsource(argc, argv)
 int argc;
-char *argv[];
+char *CONST argv[];
 {
 	int ret;
 
@@ -3252,7 +3422,7 @@ char *argv[];
 #endif	/* _NOORIGSHELL */
 
 int checkbuiltin(comname)
-char *comname;
+CONST char *comname;
 {
 	int i;
 
@@ -3263,7 +3433,7 @@ char *comname;
 }
 
 int checkinternal(comname)
-char *comname;
+CONST char *comname;
 {
 	int i;
 
@@ -3275,7 +3445,7 @@ char *comname;
 
 int execbuiltin(n, argc, argv)
 int n, argc;
-char *argv[];
+char *CONST argv[];
 {
 	return(((*builtinlist[n].func)(argc, argv) < 0)
 		? RET_NOTICE : RET_SUCCESS);
@@ -3283,7 +3453,7 @@ char *argv[];
 
 int execinternal(n, argc, argv)
 int n, argc;
-char *argv[];
+char *CONST argv[];
 {
 	if (dumbterm > 1) {
 		builtinerror(argv, NULL, ER_NOTDUMBTERM);
@@ -3315,7 +3485,7 @@ char *argv[];
 
 #ifdef	_NOORIGSHELL
 int execpseudoshell(command, flags)
-char *command;
+CONST char *command;
 int flags;
 {
 	char *cp, **argv;
@@ -3379,7 +3549,6 @@ int flags;
 		i = argc;
 		if ((cp = getenvval(&i, argv)) != (char *)-1 && i == argc) {
 			if (setenv2(argv[0], cp, 0) < 0) error(argv[0]);
-			evalenv();
 			if (cp) free(cp);
 			n = RET_SUCCESS;
 		}
@@ -3393,7 +3562,7 @@ int flags;
 
 #ifndef	_NOCOMPLETE
 int completebuiltin(com, len, argc, argvp)
-char *com;
+CONST char *com;
 int len, argc;
 char ***argvp;
 {
@@ -3412,7 +3581,7 @@ char ***argvp;
 }
 
 int completeinternal(com, len, argc, argvp)
-char *com;
+CONST char *com;
 int len, argc;
 char ***argvp;
 {
