@@ -9,6 +9,10 @@
 #include "termio.h"
 #include "func.h"
 
+#ifndef	O_ACCMODE
+#define	O_ACCMODE	(O_RDONLY | O_WRONLY | O_RDWR)
+#endif
+
 #if	!defined (_NODOSDRIVE) && defined (CODEEUC)
 extern int noconv;
 #endif
@@ -51,7 +55,7 @@ static int maxdirpath = 0;
 
 #ifndef	_NODOSDRIVE
 int _dospath(path)
-char *path;
+CONST char *path;
 {
 	if (!dosdrive) return(0);
 
@@ -59,9 +63,11 @@ char *path;
 }
 
 int dospath(path, buf)
-char *path, *buf;
+CONST char *path;
+char *buf;
 {
-	char *cp, tmp[MAXPATHLEN];
+	CONST char *cp;
+	char *s, tmp[MAXPATHLEN];
 	int drive, len;
 
 	if (!dosdrive) return(0);
@@ -71,7 +77,7 @@ char *path, *buf;
 
 	drive = *cp;
 	if (!buf) return(drive);
-	len = MAXPATHLEN - 1;
+	len = sizeof(tmp) - 1;
 	if (cp == buf) {
 		strncpy2(tmp, cp, len);
 		cp = tmp;
@@ -84,13 +90,13 @@ char *path, *buf;
 # endif
 	strncpy2(buf, cp, len);
 	if (cp != path && *path) {
-		cp = strcatdelim(buf);
-		len -= cp - buf;
+		s = strcatdelim(buf);
+		len -= s - buf;
 # ifdef	CODEEUC
-		if (!noconv) cp[ujis2sjis(cp, (u_char *)path, len)] = '\0';
+		if (!noconv) s[ujis2sjis(s, (u_char *)path, len)] = '\0';
 		else
 # endif
-		strncpy2(cp, path, len);
+		strncpy2(s, path, len);
 	}
 
 	return(drive);
@@ -98,13 +104,14 @@ char *path, *buf;
 #endif	/* !_NODOSDRIVE */
 
 DIR *Xopendir(path)
-char *path;
+CONST char *path;
 {
 #if	!defined (_NOKANJIFCONV) || !defined (_NOROCKRIDGE) || defined (CYGWIN)
 	char buf[MAXPATHLEN];
 #endif
 	DIR *dirp;
-	char *cp, conv[MAXPATHLEN];
+	CONST char *cp;
+	char conv[MAXPATHLEN];
 
 	cp = convput(conv, path, 1, 1, NULL, NULL);
 #ifndef	_NODOSDRIVE
@@ -252,14 +259,14 @@ DIR *dirp;
 	if (!dp) return(NULL);
 
 	dest = ((struct dirent *)&buf) -> d_name;
-#ifdef	CYGWIN
+# ifdef	CYGWIN
 	/* Some versions of Cygwin have neither d_fileno nor d_ino */
 	if (dos) {
 		src = ((struct dosdirent *)dp) -> d_name;
-		buf.d_reclen = ((struct dosdirent *)dp) -> d_reclen;
+		wrap_reclen(&buf) = ((struct dosdirent *)dp) -> d_reclen;
 	}
 	else
-#endif
+# endif
 	{
 		src = dp -> d_name;
 		memcpy((char *)(&buf), (char *)dp, dest - (char *)&buf);
@@ -303,7 +310,7 @@ DIR *dirp;
 }
 
 int Xchdir(path)
-char *path;
+CONST char *path;
 {
 #ifndef	_NODOSDRIVE
 	int dd, drive;
@@ -388,7 +395,7 @@ char *path;
 }
 
 int Xstat(path, stp)
-char *path;
+CONST char *path;
 struct stat *stp;
 {
 	char conv[MAXPATHLEN];
@@ -405,7 +412,7 @@ struct stat *stp;
 }
 
 int Xlstat(path, stp)
-char *path;
+CONST char *path;
 struct stat *stp;
 {
 #ifndef	_NOROCKRIDGE
@@ -414,7 +421,11 @@ struct stat *stp;
 	char conv[MAXPATHLEN];
 	int n;
 
+#ifdef	_NOROCKRIDGE
+	path = convput(conv, path, 1, 0, NULL, NULL);
+#else
 	path = convput(conv, path, 1, 0, rpath, NULL);
+#endif
 #ifndef	_NODOSDRIVE
 	if (_dospath(path)) n = doslstat(path, stp);
 	else
@@ -428,7 +439,7 @@ struct stat *stp;
 }
 
 int Xaccess(path, mode)
-char *path;
+CONST char *path;
 int mode;
 {
 	char conv[MAXPATHLEN];
@@ -445,7 +456,7 @@ int mode;
 }
 
 int Xsymlink(name1, name2)
-char *name1, *name2;
+CONST char *name1, *name2;
 {
 	char conv1[MAXPATHLEN], conv2[MAXPATHLEN];
 	int n;
@@ -463,7 +474,8 @@ char *name1, *name2;
 }
 
 int Xreadlink(path, buf, bufsiz)
-char *path, *buf;
+CONST char *path;
+char *buf;
 int bufsiz;
 {
 	char conv[MAXPATHLEN], lbuf[MAXPATHLEN + 1];
@@ -471,7 +483,8 @@ int bufsiz;
 
 	path = convput(conv, path, 1, 0, lbuf, &c);
 #ifndef	_NOROCKRIDGE
-	if (*lbuf && (n = rrreadlink(lbuf, lbuf, MAXPATHLEN)) >= 0) /*EMPTY*/;
+	if (*lbuf && (n = rrreadlink(lbuf, lbuf, sizeof(lbuf) - 1)) >= 0)
+		/*EMPTY*/;
 	else
 #endif
 #ifndef	_NODOSDRIVE
@@ -483,7 +496,7 @@ int bufsiz;
 		lbuf[n] = '\0';
 #ifndef	_NOKANJIFCONV
 		path = kanjiconv2(conv, lbuf,
-			MAXPATHLEN - 1, c, DEFCODE, L_FNAME);
+			strsize(conv), c, DEFCODE, L_FNAME);
 #else
 		path = lbuf;
 #endif
@@ -494,7 +507,7 @@ int bufsiz;
 }
 
 int Xchmod(path, mode)
-char *path;
+CONST char *path;
 int mode;
 {
 	char conv[MAXPATHLEN];
@@ -513,8 +526,8 @@ int mode;
 
 #ifdef	USEUTIME
 int Xutime(path, times)
-char *path;
-struct utimbuf *times;
+CONST char *path;
+CONST struct utimbuf *times;
 {
 	char conv[MAXPATHLEN];
 	int n;
@@ -531,8 +544,8 @@ struct utimbuf *times;
 }
 #else	/* !USEUTIME */
 int Xutimes(path, tvp)
-char *path;
-struct timeval tvp[2];
+CONST char *path;
+CONST struct timeval *tvp;
 {
 	char conv[MAXPATHLEN];
 	int n;
@@ -551,7 +564,7 @@ struct timeval tvp[2];
 
 #ifdef	HAVEFLAGS
 int Xchflags(path, flags)
-char *path;
+CONST char *path;
 u_long flags;
 {
 	char conv[MAXPATHLEN];
@@ -574,7 +587,7 @@ u_long flags;
 
 #ifndef	NOUID
 int Xchown(path, uid, gid)
-char *path;
+CONST char *path;
 uid_t uid;
 gid_t gid;
 {
@@ -597,7 +610,7 @@ gid_t gid;
 #endif	/* !NOUID */
 
 int Xunlink(path)
-char *path;
+CONST char *path;
 {
 	char conv[MAXPATHLEN];
 	int n;
@@ -614,7 +627,7 @@ char *path;
 }
 
 int Xrename(from, to)
-char *from, *to;
+CONST char *from, *to;
 {
 	char conv1[MAXPATHLEN], conv2[MAXPATHLEN];
 	int n;
@@ -642,7 +655,7 @@ char *from, *to;
 }
 
 int Xopen(path, flags, mode)
-char *path;
+CONST char *path;
 int flags, mode;
 {
 	char conv[MAXPATHLEN];
@@ -701,7 +714,7 @@ int nbytes;
 
 int Xwrite(fd, buf, nbytes)
 int fd;
-char *buf;
+CONST char *buf;
 int nbytes;
 {
 	int n;
@@ -725,6 +738,18 @@ int whence;
 	return(ofs);
 }
 
+int Xftruncate(fd, len)
+int fd;
+off_t len;
+{
+	int n;
+
+	if (fd >= DOSFDOFFSET) n = dosftruncate(fd, len);
+	else n = ftruncate(fd, len);
+
+	return(n);
+}
+
 int Xdup(oldd)
 int oldd;
 {
@@ -756,7 +781,7 @@ int oldd, newd;
 #endif	/* !_NODOSDRIVE */
 
 int Xmkdir(path, mode)
-char *path;
+CONST char *path;
 int mode;
 {
 	char conv[MAXPATHLEN];
@@ -774,7 +799,7 @@ int mode;
 }
 
 int Xrmdir(path)
-char *path;
+CONST char *path;
 {
 	char conv[MAXPATHLEN];
 	int n;
@@ -791,7 +816,7 @@ char *path;
 }
 
 FILE *Xfopen(path, type)
-char *path, *type;
+CONST char *path, *type;
 {
 	FILE *fp;
 	char conv[MAXPATHLEN];
@@ -811,7 +836,7 @@ char *path, *type;
 #ifndef	_NODOSDRIVE
 FILE *Xfdopen(fd, type)
 int fd;
-char *type;
+CONST char *type;
 {
 	FILE *fp;
 
@@ -868,7 +893,7 @@ FILE *stream;
 }
 
 int Xfwrite(buf, size, nitems, stream)
-char *buf;
+CONST char *buf;
 int size, nitems;
 FILE *stream;
 {
@@ -928,7 +953,7 @@ FILE *stream;
 }
 
 int Xfputs(s, stream)
-char *s;
+CONST char *s;
 FILE *stream;
 {
 	int n;
