@@ -10,15 +10,11 @@
 #include "func.h"
 #include "kanji.h"
 
-#ifndef	ENOSPC
-#define	ENOSPC	EACCES
-#endif
-
 #if	MSDOS
 #include <process.h>
 extern int getcurdrv __P_((VOID_A));
 # ifndef	_NOUSELFN
-extern char *preparefile __P_((char *, char *));
+extern char *preparefile __P_((CONST char *, char *));
 # endif
 #else
 #include <pwd.h>
@@ -42,9 +38,7 @@ extern int dosdrive;
 #endif
 
 #define	MAXTMPNAMLEN	8
-#ifndef	O_BINARY
-#define	O_BINARY	0
-#endif
+#define	LOCKEXT		"LCK"
 #ifdef	_NODOSDRIVE
 #define	DOSDIRENT	32
 #define	LFNENTSIZ	13
@@ -53,6 +47,15 @@ extern int dosdrive;
 #define	DOSBODYLEN	8
 #define	DOSEXTLEN	3
 
+#ifndef	O_BINARY
+#define	O_BINARY	0
+#endif
+#ifndef	O_ACCMODE
+#define	O_ACCMODE	(O_RDONLY | O_WRONLY | O_RDWR)
+#endif
+#ifndef	ENOSPC
+#define	ENOSPC	EACCES
+#endif
 #ifndef	L_SET
 # ifdef	SEEK_SET
 # define	L_SET	SEEK_SET
@@ -61,31 +64,36 @@ extern int dosdrive;
 # endif
 #endif	/* !L_SET */
 
+#ifndef	NOFLOCK
+static int NEAR fcntllock __P_((CONST char *, int, int));
+#endif
+static char *NEAR excllock __P_((CONST char *, int));
 #ifdef	_NODOSDRIVE
 #define	nodoschdir	Xchdir
 #define	nodosgetwd	Xgetwd
 #define	nodosmkdir	Xmkdir
 #define	nodosrmdir	Xrmdir
 #else
-static int NEAR nodoschdir __P_((char *));
+static int NEAR nodoschdir __P_((CONST char *));
 static char *NEAR nodosgetwd __P_((char *));
-static int NEAR nodosmkdir __P_((char *, int));
-static int NEAR nodosrmdir __P_((char *));
+static int NEAR nodosmkdir __P_((CONST char *, int));
+static int NEAR nodosrmdir __P_((CONST char *));
 #endif
-static int NEAR cpfile __P_((char *, char *, struct stat *, struct stat *));
+static int NEAR cpfile __P_((CONST char *, CONST char *,
+		struct stat *, struct stat *));
 static VOID changemes __P_((VOID_A));
 static int NEAR genrand __P_((int));
-static int dounlink __P_((char *));
-static int dormdir __P_((char *));
+static int dounlink __P_((CONST char *));
+static int dormdir __P_((CONST char *));
 #ifndef	_NOWRITEFS
-static int NEAR isexist __P_((char *));
-static int NEAR realdirsiz __P_((char *, int, int, int, int));
+static int NEAR isexist __P_((CONST char *));
+static int NEAR realdirsiz __P_((CONST char *, int, int, int, int));
 static int NEAR getnamlen __P_((int, int, int, int, int));
-static int NEAR saferename __P_((char *, char *));
-static char *NEAR maketmpfile __P_((int, int, char *, char *));
+static int NEAR saferename __P_((CONST char *, CONST char *));
+static char *NEAR maketmpfile __P_((int, int, CONST char *, CONST char *));
 #if	!MSDOS
-static off_t NEAR getdirblocksize __P_((char *));
-static u_char *NEAR getentnum __P_((char *, off_t));
+static off_t NEAR getdirblocksize __P_((CONST char *));
+static u_char *NEAR getentnum __P_((CONST char *, off_t));
 #endif
 static VOID NEAR restorefile __P_((char *, char *, int));
 #endif	/* !_NOWRITEFS */
@@ -102,8 +110,9 @@ static int dosdrv = -1;
 
 
 #ifdef	_USEDOSEMU
-char *nodospath(path, file)
-char *path, *file;
+CONST char *nodospath(path, file)
+char *path;
+CONST char *file;
 {
 	if (!_dospath(file)) return(file);
 	path[0] = '.';
@@ -146,7 +155,7 @@ namelist *namep;
 	char path[MAXPATHLEN];
 #endif
 	struct stat st, lst;
-	char *cp;
+	CONST char *cp;
 
 	cp = nodospath(path, namep -> name);
 	if (Xlstat(cp, &lst) < 0 || stat2(cp, &st) < 0) return(-1);
@@ -275,22 +284,22 @@ int cmptree(vp1, vp2)
 CONST VOID_P vp1;
 CONST VOID_P vp2;
 {
-	char *cp1, *cp2;
+	treelist *tp1, *tp2;
+	CONST char *cp1, *cp2;
 	int tmp;
 
-	if (!(((treelist *)vp1) -> name)) return(1);
-	if (!(((treelist *)vp2) -> name)) return(-1);
+	tp1 = (treelist *)vp1;
+	tp2 = (treelist *)vp2;
+	if (!(tp1 -> name)) return(1);
+	if (!(tp2 -> name)) return(-1);
 	switch (sorton & 7) {
 		case 1:
-			tmp = strpathcmp2(((treelist *)vp1) -> name,
-				((treelist *)vp2) -> name);
+			tmp = strpathcmp2(tp1 -> name, tp2 -> name);
 			break;
 		case 2:
-			if ((cp1 = strrchr(((treelist *)vp1) -> name, '.')))
-				cp1++;
+			if ((cp1 = strrchr(tp1 -> name, '.'))) cp1++;
 			else cp1 = nullstr;
-			if ((cp2 = strrchr(((treelist *)vp2) -> name, '.')))
-				cp2++;
+			if ((cp2 = strrchr(tp2 -> name, '.'))) cp2++;
 			else cp2 = nullstr;
 			tmp = strpathcmp2(cp1, cp2);
 			break;
@@ -304,13 +313,13 @@ CONST VOID_P vp2;
 
 	return(tmp);
 }
-#endif
+#endif	/* !_NOTREE */
 
 /*ARGSUSED*/
 struct dirent *searchdir(dirp, regexp, arcregstr)
 DIR *dirp;
-reg_t *regexp;
-char *arcregstr;
+CONST reg_t *regexp;
+CONST char *arcregstr;
 {
 	struct dirent *dp;
 #ifndef	_NOARCHIVE
@@ -343,7 +352,8 @@ int underhome(buf)
 char *buf;
 {
 	static char *homedir = NULL;
-	char *cp, cwd[MAXPATHLEN];
+	CONST char *cp;
+	char cwd[MAXPATHLEN];
 	int len;
 
 #if	MSDOS
@@ -382,11 +392,11 @@ char *buf;
 }
 
 int preparedir(dir)
-char *dir;
+CONST char *dir;
 {
 	struct stat st;
 	char tmp[MAXPATHLEN];
-	char *cp;
+	CONST char *cp;
 
 	cp = dir;
 #ifdef	_USEDOSPATH
@@ -420,7 +430,8 @@ char *dir;
 }
 
 #ifndef	NOFLOCK
-int lockfile(fd, mode)
+static int NEAR fcntllock(path, fd, mode)
+CONST char *path;
 int fd, mode;
 {
 	static int lockmode[] = {
@@ -428,37 +439,256 @@ int fd, mode;
 		F_RDLCK, F_WRLCK, F_UNLCK,
 # else	/* !USEFCNTLOCK */
 #  ifdef	USELOCKF
-		F_LOCK, F_LOCK, F_ULOCK,
+		F_TLOCK, F_TLOCK, F_ULOCK,
 #  else
-		LOCK_SH, LOCK_EX, LOCK_UN,
+		LOCK_SH | LOCK_NB, LOCK_EX | LOCK_NB, LOCK_UN,
 #  endif
 # endif	/* !USEFCNTLOCK */
 	};
 # ifdef	USEFCNTLOCK
 	struct flock lock;
 # endif
+	int i, n;
 
 # ifndef	_NODOSDRIVE
 	if (fd >= DOSFDOFFSET) return(0);
 # endif
 
+	n = -1;
+	errno = 0;
+	for (i = 0; i < LCK_MAXRETRY; i++) {
 # ifdef	USEFCNTLOCK
-	lock.l_type = lockmode[mode];
-	lock.l_start = lock.l_len = (off_t)0;
-	lock.l_whence = SEEK_SET;
-	return(fcntl(fd, F_SETLKW, &lock));
+		lock.l_type = lockmode[mode];
+		lock.l_start = lock.l_len = (off_t)0;
+		lock.l_whence = SEEK_SET;
+		n = fcntl(fd, F_SETLK, &lock);
 # else	/* !USEFCNTLOCK */
 #  ifdef	USELOCKF
-	return(lockf(fd, lockmode[mode], (off_t)0));
+		n = lockf(fd, lockmode[mode], (off_t)0);
 #  else
-	return(flock(fd, lockmode[mode]));
+		n = flock(fd, lockmode[mode]);
 #  endif
 # endif	/* !USEFCNTLOCK */
+		if (n >= 0) break;
+# ifdef	EACCES
+		else if (errno == EACCES) /*EMPTY*/;
+# endif
+# ifdef	EAGAIN
+		else if (errno == EAGAIN) /*EMPTY*/;
+# endif
+# ifdef	EWOULDBLOCK
+		else if (errno == EWOULDBLOCK) /*EMPTY*/;
+# endif
+		else break;
+
+		if (intrkey(-1)) break;
+# if	!MSDOS || defined (DJGPP)
+                usleep(100000L);
+# endif
+	}
+	if (i >= LCK_MAXRETRY) {
+#ifdef	ETIMEDOUT
+		errno = ETIMEDOUT;
+#else
+		errno = EEXIST;
+#endif
+		if (isttyiomode) warning(-1, path);
+		else perror2(path);
+	}
+
+	return(n);
 }
 #endif	/* !NOFLOCK */
 
+static char *NEAR excllock(file, mode)
+CONST char *file;
+int mode;
+{
+#if	MSDOS
+	char *ext;
+#endif
+	static char **locklist = NULL;
+	char *cp, path[MAXPATHLEN];
+	int i, fd;
+
+	if (!file) {
+		if (locklist) {
+			for (i = 0; locklist[i]; i++) {
+				VOID_C unlink(locklist[i]);
+				free(locklist[i]);
+			}
+			free(locklist);
+		}
+		return(NULL);
+	}
+
+	if (mode == LCK_UNLOCK) {
+		VOID_C Xunlink(file);
+		if (locklist) {
+			for (i = 0; locklist[i]; i++)
+				if (file == locklist[i]) break;
+			if (locklist[i]) {
+				for (; locklist[i + 1]; i++)
+					locklist[i] = locklist[i + 1];
+				locklist[i] = NULL;
+			}
+			if (!i) {
+				free(locklist);
+				locklist = NULL;
+			}
+		}
+		return(NULL);
+	}
+
+#if	MSDOS
+	cp = getbasename(file);
+	for (i = 0; cp[i]; i++) if (cp[i] == '.') break;
+	ext = &(cp[i]);
+	if (i > DOSBODYLEN - strsize(LOCKEXT))
+		i = DOSBODYLEN - strsize(LOCKEXT);
+	i += cp - file;
+	snprintf2(path, sizeof(path), "%-.*s%s%s", i, file, LOCKEXT, ext);
+#else
+	snprintf2(path, sizeof(path), "%s.%s", file, LOCKEXT);
+#endif
+
+	fd = -1;
+	for (i = 0; i < LCK_MAXRETRY; i++) {
+		fd = Xopen(path, O_BINARY | O_WRONLY | O_CREAT | O_EXCL,
+			0666 & ~tmpumask);
+		if (fd >= 0 || errno != EEXIST) break;
+
+		if (intrkey(-1)) break;
+#if	!MSDOS || defined (DJGPP)
+		usleep(100000L);
+#endif
+	}
+	if (i >= LCK_MAXRETRY) {
+#ifdef	ETIMEDOUT
+		errno = ETIMEDOUT;
+#else
+		errno = EEXIST;
+#endif
+		if (isttyiomode) warning(-1, path);
+		else perror2(path);
+	}
+	if (fd < 0) return(NULL);
+
+	Xclose(fd);
+	cp = strdup2(path);
+	i = countvar(locklist);
+	locklist = (char **)realloc2(locklist, (i + 2) * sizeof(char *));
+	locklist[i] = cp;
+	locklist[++i] = NULL;
+
+	return(cp);
+}
+
+lockbuf_t *lockopen(path, flags, mode)
+CONST char *path;
+int flags, mode;
+{
+	char *lckname;
+	lockbuf_t *lck;
+	int fd, lckflags, lckmode, duperrno;
+
+#ifdef	FAKEUNINIT
+	fd = -1;	/* fake for -Wuninitialized */
+#endif
+	lckname = NULL;
+	lckflags = 0;
+	lckmode = ((flags & O_ACCMODE) == O_RDONLY) ? LCK_READ : LCK_WRITE;
+
+#ifndef	NOFLOCK
+	if (isnfs(path) <= 0) {
+		lckflags |= LCK_FLOCK;
+		fd = newdup(Xopen(path, flags & ~O_TRUNC, mode));
+		if (fd < 0) {
+			if ((flags & O_ACCMODE) == O_WRONLY || errno != ENOENT)
+				return(NULL);
+		}
+		else if (fcntllock(path, fd, lckmode) < 0) {
+			Xclose(fd);
+			lckflags &= ~LCK_FLOCK;
+		}
+		else if ((flags & O_TRUNC) && Xftruncate(fd, (off_t)0) < 0) {
+			duperrno = errno;
+			VOID_C fcntllock(path, fd, LCK_UNLOCK);
+			Xclose(fd);
+			errno = duperrno;
+			return(NULL);
+		}
+	}
+#endif	/* !NOFLOCK */
+
+	if (!(lckflags & LCK_FLOCK)) {
+		if ((lckname = excllock(path, lckmode))) /*EMPTY*/;
+		else if ((flags & O_ACCMODE) != O_RDONLY) return(NULL);
+
+		if ((fd = newdup(Xopen(path, flags, mode))) >= 0) /*EMPTY*/;
+		else if ((flags & O_ACCMODE) == O_WRONLY || errno != ENOENT) {
+			duperrno = errno;
+			VOID_C excllock(lckname, LCK_UNLOCK);
+			free(lckname);
+			errno = duperrno;
+			return(NULL);
+		}
+	}
+
+	if (fd < 0) lckflags |= LCK_INVALID;
+	lck = (lockbuf_t *)malloc2(sizeof(lockbuf_t));
+	lck -> fd = fd;
+	lck -> fp = NULL;
+	lck -> name = lckname;
+	lck -> flags = lckflags;
+
+	return(lck);
+}
+
+lockbuf_t *lockfopen(path, type, flags)
+CONST char *path, *type;
+int flags;
+{
+	lockbuf_t *lck;
+	FILE *fp;
+
+	if (!(lck = lockopen(path, flags, 0666))) return(NULL);
+
+	if (!(lck -> flags & LCK_INVALID)) {
+		if (!(fp = Xfdopen(lck -> fd, type))) {
+			lockclose(lck);
+			return(NULL);
+		}
+		lck -> fp = fp;
+	}
+	lck -> flags |= LCK_STREAM;
+
+	return(lck);
+}
+
+VOID lockclose(lck)
+lockbuf_t *lck;
+{
+	if (lck) {
+		if (lck -> name) {
+			VOID_C excllock(lck -> name, LCK_UNLOCK);
+			free(lck -> name);
+		}
+		if (!(lck -> flags & LCK_INVALID)) {
+#ifndef	NOFLOCK
+			if (lck -> flags & LCK_FLOCK)
+				VOID_C fcntllock(NULL, lck -> fd, LCK_UNLOCK);
+#endif
+			if (lck -> flags & LCK_STREAM) Xfclose(lck -> fp);
+			else Xclose(lck -> fd);
+		}
+
+		free(lck);
+	}
+}
+
 int touchfile(path, stp)
-char *path;
+CONST char *path;
 struct stat *stp;
 {
 #ifndef	_NOEXTRAATTR
@@ -560,6 +790,7 @@ struct stat *stp;
 		}
 	}
 #endif	/* !NOUID */
+	if (stp -> st_nlink & TCH_IGNOREERR) ret = 0;
 	if (ret < 0) errno = duperrno;
 
 	return(ret);
@@ -567,7 +798,7 @@ struct stat *stp;
 
 #ifndef	_NODOSDRIVE
 static int NEAR nodoschdir(path)
-char *path;
+CONST char *path;
 {
 	int n, dupdosdrive;
 
@@ -594,7 +825,7 @@ char *path;
 }
 
 int nodoslstat(path, stp)
-char *path;
+CONST char *path;
 struct stat *stp;
 {
 	int n, dupdosdrive;
@@ -608,7 +839,7 @@ struct stat *stp;
 }
 
 static int NEAR nodosmkdir(path, mode)
-char *path;
+CONST char *path;
 int mode;
 {
 	int n, dupdosdrive;
@@ -622,7 +853,7 @@ int mode;
 }
 
 static int NEAR nodosrmdir(path)
-char *path;
+CONST char *path;
 {
 	int n, dupdosdrive;
 
@@ -638,7 +869,8 @@ char *path;
 VOID lostcwd(path)
 char *path;
 {
-	char *cp, buf[MAXPATHLEN];
+	CONST char *cp;
+	char buf[MAXPATHLEN];
 	int duperrno;
 
 	duperrno = errno;
@@ -661,7 +893,7 @@ char *path;
 
 #ifndef	NODIRLOOP
 int issamebody(src, dest)
-char *src, *dest;
+CONST char *src, *dest;
 {
 	struct stat st1, st2;
 
@@ -673,7 +905,7 @@ char *src, *dest;
 
 #ifndef	NOSYMLINK
 int cpsymlink(src, dest)
-char *src, *dest;
+CONST char *src, *dest;
 {
 	struct stat st;
 	char path[MAXPATHLEN];
@@ -695,7 +927,7 @@ char *src, *dest;
 
 /*ARGSUSED*/
 static int NEAR cpfile(src, dest, stp1, stp2)
-char *src, *dest;
+CONST char *src, *dest;
 struct stat *stp1, *stp2;
 {
 #if	MSDOS
@@ -743,6 +975,9 @@ struct stat *stp1, *stp2;
 			duperrno = errno;
 			break;
 		}
+#ifndef	_NOEXTRACOPY
+		showprogress(i);
+#endif
 		if (surewrite(fd2, buf, i) < 0) {
 			i = -1;
 			duperrno = errno;
@@ -753,13 +988,16 @@ struct stat *stp1, *stp2;
 
 	VOID_C Xclose(fd2);
 	VOID_C Xclose(fd1);
+#ifndef	_NOEXTRACOPY
+	fshowprogress(dest);
+#endif
 
 	if (i < 0) {
-		Xunlink(dest);
+		VOID_C Xunlink(dest);
 		errno = duperrno;
 		return(-1);
 	}
-	stp1 -> st_nlink = (TCH_ATIME | TCH_MTIME);
+	stp1 -> st_nlink = (TCH_ATIME | TCH_MTIME | TCH_IGNOREERR);
 #ifdef	_USEDOSCOPY
 	if (touchfile(dest, stp1) < 0) return(-1);
 #else
@@ -777,7 +1015,7 @@ static VOID changemes(VOID_A)
 }
 
 int safecpfile(src, dest, stp1, stp2)
-char *src, *dest;
+CONST char *src, *dest;
 struct stat *stp1, *stp2;
 {
 #ifndef	_NODOSDRIVE
@@ -807,16 +1045,21 @@ struct stat *stp1, *stp2;
 }
 
 int safemvfile(src, dest, stp1, stp2)
-char *src, *dest;
+CONST char *src, *dest;
 struct stat *stp1, *stp2;
 {
-	if (Xrename(src, dest) >= 0) return(0);
+	if (Xrename(src, dest) >= 0) {
+#ifndef	_NOEXTRACOPY
+		fshowprogress(dest);
+#endif
+		return(0);
+	}
 	if (errno != EXDEV || s_isdir(stp1)) return(-1);
 	if (safecpfile(src, dest, stp1, stp2) < 0 || Xunlink(src) < 0)
 		return(-1);
 
 	stp1 -> st_nlink = (TCH_MODE | TCH_UID | TCH_GID
-		| TCH_ATIME | TCH_MTIME);
+		| TCH_ATIME | TCH_MTIME | TCH_IGNOREERR);
 	return (touchfile(dest, stp1));
 }
 
@@ -897,8 +1140,7 @@ char *dir;
 #endif
 	deftmpdir = strdup2(path);
 #if	MSDOS
-	n = getcurdrv();
-	*path = (isupper2(n)) ? toupper2(*path) : tolower2(*path);
+	*path = (isupper2(getcurdrv())) ? toupper2(*path) : tolower2(*path);
 #endif
 
 	mask = 0777 & ~tmpumask;
@@ -957,7 +1199,7 @@ char *dir;
 }
 
 int rmtmpdir(dir)
-char *dir;
+CONST char *dir;
 {
 	char path[MAXPATHLEN];
 
@@ -977,28 +1219,40 @@ char *dir;
 	return(0);
 }
 
-int mktmpfile(file)
-char *file;
+int opentmpfile(path, mode)
+CONST char *path;
+int mode;
 {
-	char *cp, path[MAXPATHLEN];
-	int fd, len, duperrno;
+	char *cp;
+	int fd, len;
 
-	path[0] = '\0';
-	if (mktmpdir(path) < 0) return(-1);
-	cp = strcatdelim(path);
-	len = strsize(path) - (cp - path);
+	cp = getbasename(path);
+	len = MAXPATHLEN - (cp - path) - 1;
 	if (len > MAXTMPNAMLEN) len = MAXTMPNAMLEN;
 	genrandname(NULL, 0);
 
 	for (;;) {
 		genrandname(cp, len);
-		fd = Xopen(path, O_BINARY | O_WRONLY | O_CREAT | O_EXCL,
-			0666 & ~tmpumask);
-		if (fd >= 0) {
-			strcpy(file, path);
-			return(fd);
-		}
-		if (errno != EEXIST) break;
+		fd = Xopen(path, O_BINARY | O_WRONLY | O_CREAT | O_EXCL, mode);
+		if (fd >= 0) break;
+		if (errno != EEXIST) return(-1);
+	}
+
+	return(fd);
+}
+
+int mktmpfile(file)
+char *file;
+{
+	char path[MAXPATHLEN];
+	int fd, duperrno;
+
+	path[0] = '\0';
+	if (mktmpdir(path) < 0) return(-1);
+	VOID_C strcatdelim(path);
+	if ((fd = opentmpfile(path, 0666 & ~tmpumask)) >= 0) {
+		strcpy(file, path);
+		return(fd);
 	}
 	duperrno = errno;
 	rmtmpdir(NULL);
@@ -1008,7 +1262,7 @@ char *file;
 }
 
 int rmtmpfile(file)
-char *file;
+CONST char *file;
 {
 	if ((Xunlink(file) < 0 && errno != ENOENT) || rmtmpdir(NULL) < 0)
 		return(-1);
@@ -1017,21 +1271,25 @@ char *file;
 }
 
 static int dounlink(path)
-char *path;
+CONST char *path;
 {
-	return(Xunlink(path));
+	if (Xunlink(path) < 0) return(APL_ERROR);
+
+	return(APL_OK);
 }
 
 static int dormdir(path)
-char *path;
+CONST char *path;
 {
-	if (isdotdir(path)) return(0);
+	if (isdotdir(path)) return(APL_OK);
+	if (Xrmdir(path) < 0) return(APL_ERROR);
 
-	return(Xrmdir(path));
+	return(APL_OK);
 }
 
 VOID removetmp(dir, file)
-char *dir, *file;
+char *dir;
+CONST char *file;
 {
 #ifdef	_USEDOSEMU
 	char path[MAXPATHLEN];
@@ -1042,7 +1300,8 @@ char *dir, *file;
 		warning(-1, dir);
 		*dir = '\0';
 	}
-	else if (!file) applydir(NULL, dounlink, NULL, dormdir, 3, NULL);
+	else if (!file) VOID_C applydir(NULL, dounlink,
+		NULL, dormdir, ORD_NOPREDIR, NULL);
 	else if (Xunlink(nodospath(path, file)) < 0) warning(-1, file);
 
 #ifndef	_NODOSDRIVE
@@ -1067,7 +1326,7 @@ char *dir, *file;
 }
 
 int forcecleandir(dir, file)
-char *dir, *file;
+CONST char *dir, *file;
 {
 #if	!MSDOS
 	p_id_t pid;
@@ -1075,6 +1334,8 @@ char *dir, *file;
 	extern char **environ;
 	char buf[MAXPATHLEN];
 
+	VOID_C excllock(NULL, LCK_UNLOCK);
+
 	if (!dir || !*dir || !file || !*file) return(0);
 	strcatdelim2(buf, dir, file);
 
@@ -1108,7 +1369,8 @@ int drive;
 }
 
 int tmpdosdupl(dir, dirp, single)
-char *dir, **dirp;
+CONST char *dir;
+char **dirp;
 int single;
 {
 	struct stat st;
@@ -1164,7 +1426,7 @@ int single;
 
 int tmpdosrestore(drive, file)
 int drive;
-char *file;
+CONST char *file;
 {
 	struct stat st;
 	char path[MAXPATHLEN];
@@ -1172,9 +1434,7 @@ char *file;
 	char tmp[MAXPATHLEN];
 # endif
 
-	path[0] = drive;
-	path[1] = ':';
-	strcpy(&(path[2]), file);
+	strcpy(gendospath(path, drive, '\0'), file);
 # if	!MSDOS
 	file = nodospath(tmp, file);
 # endif
@@ -1188,7 +1448,7 @@ char *file;
 
 #ifndef	_NOWRITEFS
 static int NEAR isexist(file)
-char *file;
+CONST char *file;
 {
 	struct stat st;
 
@@ -1198,7 +1458,7 @@ char *file;
 }
 
 static int NEAR realdirsiz(s, dos, boundary, dirsize, ofs)
-char *s;
+CONST char *s;
 int dos, boundary, dirsize, ofs;
 {
 	int i, len, lfn, dot;
@@ -1252,7 +1512,7 @@ int size, dos, boundary, dirsize, ofs;
 }
 
 static int NEAR saferename(from, to)
-char *from, *to;
+CONST char *from, *to;
 {
 #ifdef	_USEDOSEMU
 	char fpath[MAXPATHLEN], tpath[MAXPATHLEN];
@@ -1270,7 +1530,7 @@ char *from, *to;
 /*ARGSUSED*/
 static char *NEAR maketmpfile(len, dos, tmpdir, old)
 int len, dos;
-char *tmpdir, *old;
+CONST char *tmpdir, *old;
 {
 	char *fname, path[MAXPATHLEN];
 	int l, fd;
@@ -1311,7 +1571,8 @@ char *tmpdir, *old;
 				}
 				else {
 					fd = Xopen(fname,
-						O_CREAT | O_EXCL, 0600);
+						O_WRONLY | O_CREAT | O_EXCL,
+						0600);
 					if (fd >= 0) {
 						VOID_C Xclose(fd);
 						return(fname);
@@ -1332,7 +1593,7 @@ char *tmpdir, *old;
 
 #if	!MSDOS
 static off_t NEAR getdirblocksize(dir)
-char *dir;
+CONST char *dir;
 {
 	struct stat st;
 
@@ -1342,7 +1603,7 @@ char *dir;
 }
 
 static u_char *NEAR getentnum(dir, bsiz)
-char *dir;
+CONST char *dir;
 off_t bsiz;
 {
 	struct stat st;
@@ -1409,7 +1670,8 @@ int fs;
 	struct dirent *dp;
 	int dos, boundary, dirsize, namofs;
 	int i, top, size, len, fnamp, ent;
-	char *cp, *tmpdir, **fnamelist, path[MAXPATHLEN];
+	CONST char *cp;
+	char *tmp, *tmpdir, **fnamelist, path[MAXPATHLEN];
 
 	switch (fs) {
 #if	!MSDOS
@@ -1520,12 +1782,12 @@ int fs;
 		else if (!strpathcmp(dp -> d_name, tmpdir)) {
 #if	MSDOS
 			if (!(dp -> d_alias[0])) len = DOSBODYLEN;
-#else
+#else	/* !MSDOS */
 # ifndef	_NODOSDRIVE
-			if (dos == 3 && dp -> d_reclen == DOSDIRENT)
+			if (dos == 3 && wrap_reclen(dp) == DOSDIRENT)
 				len = DOSBODYLEN;
 # endif
-#endif
+#endif	/* !MSDOS */
 			ent = i;
 		}
 		else {
@@ -1544,7 +1806,7 @@ int fs;
 	Xclosedir(dirp);
 
 	if (ent > 0) {
-		if (!(cp = maketmpfile(len, dos, tmpdir, tmpdir))) {
+		if (!(tmp = maketmpfile(len, dos, tmpdir, tmpdir))) {
 			warning(-1, tmpdir);
 			restorefile(tmpdir, path, fnamp);
 			freevar(fnamelist);
@@ -1552,7 +1814,7 @@ int fs;
 			return;
 		}
 		free(tmpdir);
-		tmpdir = cp;
+		tmpdir = tmp;
 		fnamp = strcatdelim2(path, tmpdir, NULL) - path;
 	}
 
@@ -1628,10 +1890,11 @@ int fs;
 	}
 #endif
 
-	if (!(cp = maketmpfile(len, dos, tmpdir, tmpdir))) warning(-1, tmpdir);
+	if (!(tmp = maketmpfile(len, dos, tmpdir, tmpdir)))
+		warning(-1, tmpdir);
 	else {
 		free(tmpdir);
-		tmpdir = cp;
+		tmpdir = tmp;
 	}
 	fnamp = strcatdelim2(path, tmpdir, NULL) - path;
 	snprintf2(&(path[fnamp]), sizeof(path) - fnamp, fnamelist[top]);
