@@ -72,12 +72,12 @@ typedef struct mnttab		mnt_t;
 || defined (USEMNTCTL) || defined (USEMNTINFOR) || defined (USEMNTINFO) \
 || defined (USEGETMNT)
 typedef struct _mnt_t {
-	char *mnt_fsname;
-	char *mnt_dir;
-	char *mnt_type;
-	char *mnt_opts;
+	CONST char *mnt_fsname;
+	CONST char *mnt_dir;
+	CONST char *mnt_type;
+	CONST char *mnt_opts;
 } mnt_t;
-static FILE *NEAR setmntent2 __P_((char *, char *));
+static FILE *NEAR setmntent2 __P_((CONST char *, CONST char *));
 static mnt_t *NEAR getmntent2 __P_((FILE *, mnt_t *));
 #define	hasmntopt2(m,o)		strmntopt((m) -> mnt_opts, o)
 # if	defined (USEMNTINFO) || defined (USEGETMNT)
@@ -103,11 +103,16 @@ typedef struct fstab		mnt_t;
 #endif	/* USEGETFSENT */
 
 #if	MSDOS
+# ifdef	DOUBLESLASH
+# define	MNTDIRSIZ	MAXPATHLEN
+# else
+# define	MNTDIRSIZ	(3 + 1)
+# endif
 typedef struct _mnt_t {
-	char *mnt_fsname;
-	char mnt_dir[4];
-	char *mnt_type;
-	char *mnt_opts;
+	CONST char *mnt_fsname;
+	char mnt_dir[MNTDIRSIZ];
+	CONST char *mnt_type;
+	CONST char *mnt_opts;
 } mnt_t;
 #define	hasmntopt2(m,o)		strmntopt((m) -> mnt_opts, o)
 # ifdef	PC98
@@ -193,24 +198,23 @@ typedef struct _statfs_t {
 	long f_bavail;
 	long f_files;
 } statfs_t;
-extern int unixstatfs __P_((char *, statfs_t *));
+extern int unixstatfs __P_((CONST char *, statfs_t *));
 #define	statfs2			unixstatfs
 #define	blocksize(fs)		(fs).f_bsize
 #endif
 
-extern VOID error __P_((char *));
-extern int _chdir2 __P_((char *));
-extern char *strcpy2 __P_((char *, char *));
-#ifndef	NOFLOCK
-extern char *strncpy2 __P_((char *, char *, int));
-#endif
+extern VOID error __P_((CONST char *));
+extern int _chdir2 __P_((CONST char *));
+extern char *strcpy2 __P_((char *, CONST char *));
+extern char *strncpy2 __P_((char *, CONST char *, int));
 extern char *getwd2 __P_((VOID_A));
-extern VOID warning __P_((int, char *));
+extern VOID warning __P_((int, CONST char *));
 #ifdef	_USEDOSPATH
-extern int dospath __P_((char *, char *));
+extern int dospath __P_((CONST char *, char *));
+extern char *gendospath __P_((char *, int, int));
 #endif
 #if	MSDOS && !defined (_NOUSELFN)
-extern int supportLFN __P_((char *));
+extern int supportLFN __P_((CONST char *));
 # ifndef	_NODOSDRIVE
 extern int checkdrive __P_((int));
 # endif
@@ -221,12 +225,12 @@ extern int dosstatfs __P_((int, char *));
 extern char *malloc2 __P_((ALLOC_T));
 extern char *realloc2 __P_((VOID_P, ALLOC_T));
 #if	!MSDOS || !defined (NOFLOCK)
-extern int Xstat __P_((char *, struct stat *));
+extern int Xstat __P_((CONST char *, struct stat *));
 #endif
-extern int Xaccess __P_((char *, int));
+extern int Xaccess __P_((CONST char *, int));
 extern int filetop __P_((int));
 extern VOID cputspace __P_((int));
-extern VOID cputstr __P_((int, char *));
+extern VOID cputstr __P_((int, CONST char *));
 #ifdef	_NOPTY
 #define	Xlocate			locate
 #define	Xputterm		putterm
@@ -237,12 +241,12 @@ extern VOID cputstr __P_((int, char *));
 extern VOID Xlocate __P_((int, int));
 extern VOID Xputterm __P_((int));
 extern VOID Xputch2 __P_((int));
-extern VOID Xcputs2 __P_((char *));
+extern VOID Xcputs2 __P_((CONST char *));
 extern VOID Xcprintf2 __P_((CONST char *, ...));
 #endif
 
 extern bindtable bindlist[];
-extern functable funclist[];
+extern CONST functable funclist[];
 extern char fullpath[];
 extern char *distributor;
 #ifndef	_NODOSDRIVE
@@ -321,19 +325,26 @@ extern int needbavail;
 #ifndef	MNTTYPE_FAT32
 #define	MNTTYPE_FAT32	"fat32"	/* Win98 */
 #endif
+#ifndef	MNTTYPE_SHARED
+#define	MNTTYPE_SHARED	"shared"	/* Win98 */
+#endif
 #define	MNTTYPE_XNFS	"nfs"	/* NFS */
 
 static int NEAR code2str __P_((char *, int));
 static int NEAR checkline __P_((int));
 VOID help __P_((int));
-static int NEAR getfsinfo __P_((char *, statfs_t *, mnt_t *));
-static char *NEAR strmntopt __P_((char *, char *));
-int writablefs __P_((char *));
-off_t getblocksize __P_((char *));
-static int NEAR info1line __P_((int, int, char *, off_t, char *, char *));
+static int NEAR getfsinfo __P_((CONST char *, statfs_t *, mnt_t *));
+static CONST char *NEAR strmntopt __P_((CONST char *, CONST char *));
+#ifndef	NOFLOCK
+int isnfs __P_((CONST char *));
+#endif
+int writablefs __P_((CONST char *));
+off_t getblocksize __P_((CONST char *));
+static int NEAR info1line __P_((int, int, CONST char *,
+		off_t, CONST char *, CONST char *));
 off_t calcKB __P_((off_t, off_t));
-int getinfofs __P_((char *, off_t *, off_t *, off_t *));
-int infofs __P_((char *));
+int getinfofs __P_((CONST char *, off_t *, off_t *, off_t *));
+int infofs __P_((CONST char *));
 
 static CONST int keycodelist[] = {
 	K_HOME, K_END, K_DL, K_IL, K_DC, K_IC,
@@ -350,6 +361,7 @@ static CONST strtable mntlist[] = {
 	{FSID_FAT, MNTTYPE_PC},
 	{FSID_LFN, MNTTYPE_DOS7},
 	{FSID_LFN, MNTTYPE_FAT32},
+	{FSID_LFN, MNTTYPE_SHARED},
 #if	MSDOS
 # ifndef	_NODOSDRIVE
 	{FSID_FAT, MNTTYPE_FAT12},
@@ -442,8 +454,8 @@ int y;
 	return(y);
 }
 
-VOID help(mode)
-int mode;
+VOID help(arch)
+int arch;
 {
 	char buf[(KEYWID + 1 + 1) * 2 + 1];
 	int i, j, c, x, y, yy;
@@ -468,7 +480,7 @@ int mode;
 		Xlocate(x * (n_column / 2), yy + y);
 		if (x ^= 1) Xputterm(L_CLEAR);
 		else y++;
-		if (mode && !(funclist[i].status & FN_ARCHIVE)) continue;
+		if (arch && !(funclist[i].status & FN_ARCHIVE)) continue;
 
 		c = 0;
 		buf[0] = '\0';
@@ -505,7 +517,7 @@ int mode;
 #ifdef	USEMNTCTL
 /*ARGSUSED*/
 static FILE *NEAR setmntent2(file, mode)
-char *file, *mode;
+CONST char *file, *mode;
 {
 	char *buf;
 
@@ -593,7 +605,7 @@ typedef struct statfs		mntinfo_t;
 
 /*ARGSUSED*/
 static FILE *NEAR setmntent2(file, mode)
-char *file, *mode;
+CONST char *file, *mode;
 {
 # ifndef	USEMNTINFO
 	int size;
@@ -712,7 +724,7 @@ mnt_t *mntp;
 #ifdef	USEGETMNT
 /*ARGSUSED*/
 static FILE *NEAR setmntent2(file, mode)
-char *file, *mode;
+CONST char *file, *mode;
 {
 	mnt_ptr = 0;
 
@@ -755,29 +767,51 @@ mnt_t *mntp;
 #endif	/* USEGETMNT */
 
 static int NEAR getfsinfo(path, fsbuf, mntbuf)
-char *path;
+CONST char *path;
 statfs_t *fsbuf;
 mnt_t *mntbuf;
 {
-	statfs_t fs;
-	mnt_t mnt;
 #if	MSDOS
 # ifndef	_NODOSDRIVE
 	int i;
 # endif
+# ifdef	DOUBLESLASH
+	char *cp;
+	int len;
+# endif
+	statfs_t fs;
+	mnt_t mnt;
+	char buf[MAXPATHLEN];
+	int drive;
 
 	if (!fsbuf) fsbuf = &fs;
 	if (!mntbuf) mntbuf = &mnt;
 
 	mntbuf -> mnt_fsname = "MSDOS";
-	mntbuf -> mnt_dir[0] = dospath(path, NULL);
-	mntbuf -> mnt_dir[1] = ':';
-	mntbuf -> mnt_dir[2] = _SC_;
-	mntbuf -> mnt_dir[3] = '\0';
+	drive = dospath(path, buf);
+# ifdef	DOUBLESLASH
+	if (drive == '_') {
+		if (*buf) path = buf;
+		len = isdslash(path);
+		if ((cp = strdelim(&(path[len]), 0))) {
+			len = cp - path;
+			if (!(cp = strdelim(&(path[len + 1]), 0)))
+				cp += strlen(cp);
+			len = cp - path;
+		}
+		strncpy2(mntbuf -> mnt_dir, path, len);
+	}
+	else
+# endif
+	VOID_C gendospath(mntbuf -> mnt_dir, drive, _SC_);
+
 # ifdef	_NOUSELFN
 	mntbuf -> mnt_type = MNTTYPE_PC;
 # else	/* !_NOUSELFN */
 	switch (supportLFN(mntbuf -> mnt_dir)) {
+		case 3:
+			mntbuf -> mnt_type = MNTTYPE_SHARED;
+			break;
 		case 2:
 			mntbuf -> mnt_type = MNTTYPE_FAT32;
 			break;
@@ -814,33 +848,36 @@ mnt_t *mntbuf;
 
 	if (statfs2(mntbuf -> mnt_dir, fsbuf) < 0) return(-1);
 #else	/* !MSDOS */
-	mnt_t *mntp;
-	FILE *fp;
-	char *dir, fsname[MAXPATHLEN];
-	ALLOC_T len, match;
 # ifndef	_NODOSDRIVE
 	int drv;
 # endif
+	statfs_t fs;
+	mnt_t mnt;
+	mnt_t *mntp;
+	FILE *fp;
+	CONST char *cp;
+	char *dir, fsname[MAXPATHLEN];
+	ALLOC_T len, match;
 
 	if (!fsbuf) fsbuf = &fs;
 	if (!mntbuf) mntbuf = &mnt;
 
-	dir = NULL;
+	cp = NULL;
 	if (!strncmp(path, "/dev/", strsize("/dev/"))) {
-		if (_chdir2(path) < 0) dir = path;
+		if (_chdir2(path) < 0) cp = path;
 	}
 # ifndef	_NODOSDRIVE
 	else if ((drv = dospath(path, NULL))) {
 		static char dosmntdir[4];
 		char buf[3 * sizeof(long) + 1];
 
-		mntbuf -> mnt_fsname = nullstr;
+		mntbuf -> mnt_fsname = (char *)nullstr;
 		mntbuf -> mnt_dir = dosmntdir;
 		dosmntdir[0] = drv;
 		strcpy(&(dosmntdir[1]), ":\\");
 		mntbuf -> mnt_type =
 			(islower2(drv)) ? MNTTYPE_DOS7 : MNTTYPE_PC;
-		mntbuf -> mnt_opts = nullstr;
+		mntbuf -> mnt_opts = (char *)nullstr;
 		if (dosstatfs(drv, buf) < 0) return(-1);
 		if (buf[3 * sizeof(long)] & 001)
 			mntbuf -> mnt_type = MNTTYPE_FAT32;
@@ -866,7 +903,7 @@ mnt_t *mntbuf;
 # endif	/* !_NODOSDRIVE */
 	else if (_chdir2(path) < 0) return(-1);
 
-	if (!dir) {
+	if (!cp) {
 		dir = getwd2();
 		if (_chdir2(fullpath) < 0) error(fullpath);
 		match = (ALLOC_T)0;
@@ -900,12 +937,12 @@ mnt_t *mntbuf;
 			errno = ENOENT;
 			return(-1);
 		}
-		dir = fsname;
+		cp = fsname;
 	}
 
 	if (!(fp = setmntent2(MOUNTED, "rb"))) return(-1);
 	while ((mntp = getmntent2(fp, &mnt)))
-		if (!strcmp(dir, mntp -> mnt_fsname)) break;
+		if (!strcmp(cp, mntp -> mnt_fsname)) break;
 	endmntent2(fp);
 	if (!mntp) {
 		errno = ENOENT;
@@ -914,17 +951,17 @@ mnt_t *mntbuf;
 	memcpy((char *)mntbuf, (char *)mntp, sizeof(mnt_t));
 
 	if (statfs2(mntbuf -> mnt_dir, fsbuf) < 0
-	&& (path == dir || statfs2(path, fsbuf) < 0))
+	&& (path == cp || statfs2(path, fsbuf) < 0))
 		return(-1);
 #endif	/* !MSDOS */
 
 	return(0);
 }
 
-static char *NEAR strmntopt(s1, s2)
-char *s1, *s2;
+static CONST char *NEAR strmntopt(s1, s2)
+CONST char *s1, *s2;
 {
-	char *cp;
+	CONST char *cp;
 	ALLOC_T len;
 
 	len = strlen(s2);
@@ -939,7 +976,7 @@ char *s1, *s2;
 
 #ifndef	NOFLOCK
 int isnfs(path)
-char *path;
+CONST char *path;
 {
 	mnt_t mntbuf;
 	struct stat st;
@@ -960,7 +997,7 @@ char *path;
 #endif	/* NOFLOCK */
 
 int writablefs(path)
-char *path;
+CONST char *path;
 {
 #ifdef	_USEDOSEMU
 	int drv;
@@ -986,7 +1023,7 @@ char *path;
 
 /*ARGSUSED*/
 off_t getblocksize(dir)
-char *dir;
+CONST char *dir;
 {
 #if	MSDOS
 	statfs_t fsbuf;
@@ -1022,9 +1059,9 @@ char *dir;
 
 static int NEAR info1line(yy, y, ind, n, s, unit)
 int yy, y;
-char *ind;
+CONST char *ind;
 off_t n;
-char *s, *unit;
+CONST char *s, *unit;
 {
 	int width;
 
@@ -1058,7 +1095,7 @@ off_t block, byte;
 }
 
 int getinfofs(path, totalp, freep, bsizep)
-char *path;
+CONST char *path;
 off_t *totalp, *freep, *bsizep;
 {
 	statfs_t fsbuf;
@@ -1071,7 +1108,10 @@ off_t *totalp, *freep, *bsizep;
 #ifndef	_NODOSDRIVE
 	needbavail--;
 #endif
-	if (n < 0) return(-1);
+	if (n < 0) {
+		*bsizep = getblocksize(path);
+		return(-1);
+	}
 
 	*totalp = fsbuf.f_blocks;
 	*freep = fsbuf.f_bavail;
@@ -1081,7 +1121,7 @@ off_t *totalp, *freep, *bsizep;
 }
 
 int infofs(path)
-char *path;
+CONST char *path;
 {
 	statfs_t fsbuf;
 	mnt_t mntbuf;
