@@ -19,15 +19,21 @@ extern short histsize[];
 extern int curcolumns;
 extern int minfilename;
 extern int hideclock;
-extern char *promptstr;
+extern CONST char *promptstr;
 #ifndef	_NOORIGSHELL
 extern int fdmode;
-extern char *promptstr2;
+extern CONST char *promptstr2;
 #endif
 #ifndef	_NOPTY
 extern int ptymode;
 extern int parentfd;
 #endif
+#ifndef	_NOIME
+extern int ime_cont;
+extern int ime_line;
+extern int *ime_xposp;
+extern VOID (*ime_locate)__P_((int, int));
+#endif
 
 #define	LIMITSELECTWARN	100
 #define	YESNOSTR	"[Y/N]"
@@ -43,16 +49,30 @@ extern int parentfd;
 #define	LEFTMARGIN	0
 #define	RIGHTMARGIN	2
 
+#ifdef	_NOKANJICONV
+#define	getinkcode()	NOCNV
+#else
+static int NEAR getinkcode __P_((VOID_A));
+#endif
 #ifndef	_NOEDITMODE
 static int NEAR getemulatekey __P_((int, CONST char []));
 #endif
-static int NEAR trquoteone __P_((char **, char *, int *, int));
-static char *NEAR trquote __P_((char *, int, int *));
-static int NEAR vlen __P_((char *, int));
-static int NEAR rlen __P_((char *, int));
-static int NEAR vonkanji1 __P_((char *, int));
+#ifndef	_NOIME
+static int NEAR getimebuf __P_((CONST char *, int *));
+static int NEAR getime __P_((int, int *, int));
+#endif
+#ifdef	_NOKANJICONV
+#define	getkey4(s)	getkey3(s, getinkcode())
+#else
+static int NEAR getkey4 __P_((int));
+#endif
+static int NEAR trquoteone __P_((char **, CONST char *, int *, int));
+static char *NEAR trquote __P_((CONST char *, int, int *));
+static int NEAR vlen __P_((CONST char *, int));
+static int NEAR rlen __P_((CONST char *, int));
+static int NEAR vonkanji1 __P_((CONST char *, int));
 #if	FD >= 2
-static VOID NEAR kanjiputs3 __P_((char *, int, int, int, int));
+static VOID NEAR kanjiputs3 __P_((CONST char *, int, int, int, int));
 #else
 #define	kanjiputs3(s,n,l,p,m)	VOID_C kanjiputs2(s, l, p)
 #endif
@@ -65,12 +85,15 @@ static VOID NEAR downcursor __P_((VOID_A));
 static VOID NEAR forwcursor __P_((int));
 static VOID NEAR backcursor __P_((int));
 static VOID NEAR forwline __P_((int));
-static VOID NEAR dumbputs __P_((char *, int, int, int, int));
+static VOID NEAR dumbputs __P_((CONST char *, int, int, int, int));
 static VOID NEAR rewritecursor __P_((int, int));
 static int NEAR checkcursor __P_((int, int));
 #endif
 static VOID NEAR scrollup __P_((VOID_A));
 static VOID NEAR locate2 __P_((int, int));
+#ifndef	_NOIME
+static VOID locate3 __P_((int, int));
+#endif
 static VOID NEAR setcursor __P_((int, int));
 static VOID NEAR putstr __P_((int *, int *, int));
 static VOID NEAR ringbell __P_((VOID_A));
@@ -88,14 +111,18 @@ static VOID NEAR insertbuf __P_((int));
 static int NEAR preparestr __P_((int, int));
 static int NEAR insertcursor __P_((int *, int *, int, int));
 static int NEAR quotemeta __P_((int *, int *, int, int *, int *, int *));
-static int NEAR insertstr __P_((char *, int, int, int *, int *));
+static int NEAR insertstr __P_((CONST char *, int, int, int *, int *));
 #ifndef	_NOCOMPLETE
-static VOID NEAR selectfile __P_((int, char **));
+static VOID NEAR selectfile __P_((int, char *CONST *));
 static int NEAR completestr __P_((int, int, int));
 #endif
+#ifndef	_NOKANJICONV
+static u_int NEAR getucs2 __P_((int));
+static VOID NEAR ungetch3 __P_((int));
+#endif
 static int NEAR getch3 __P_((VOID_A));
 static int NEAR getkanjikey __P_((char *, int));
-static VOID NEAR copyhist __P_((char *, int));
+static VOID NEAR copyhist __P_((CONST char *, int));
 static VOID NEAR _inputstr_up __P_((int *, int, char **));
 static VOID NEAR _inputstr_down __P_((int *, int, char **));
 static VOID NEAR _inputstr_delete __P_((VOID_A));
@@ -106,15 +133,16 @@ static int NEAR search_matchlen __P_((VOID_A));
 static char *NEAR search_up __P_((int, int *, int, char **));
 static char *NEAR search_down __P_((int, int *, int, char **));
 #endif
-static VOID NEAR _inputstr_input __P_((char *, int));
+static VOID NEAR _inputstr_input __P_((CONST char *, int));
 static int NEAR _inputstr __P_((int, int, int));
-static VOID NEAR dispprompt __P_((char *, int));
-static char *NEAR truncstr __P_((char *));
-static int NEAR yesnomes __P_((char *));
-static int NEAR selectcnt __P_((int, char **, int));
-static int NEAR selectadj __P_((int, int, char **, char **, int [], int));
+static VOID NEAR dispprompt __P_((CONST char *, int));
+static VOID NEAR truncstr __P_((CONST char *));
+static int NEAR yesnomes __P_((CONST char *));
+static int NEAR selectcnt __P_((int, char *CONST *, int));
+static int NEAR selectadj __P_((int, int, CONST char *CONST *,
+		char **, int *, int));
 static VOID NEAR selectmes __P_((int, int, int,
-		char *[], int [], int [], int));
+		char *CONST [], int [], int *, int));
 
 int subwindow = 0;
 int win_x = 0;
@@ -128,6 +156,9 @@ int maxcmdline = 0;
 #ifndef	_NOORIGSHELL
 int dumbshell = 0;
 #endif
+#ifndef	_NOIME
+int imekey = -1;
+#endif
 
 #ifndef	_NOEDITMODE
 static CONST int emulatekey[] = {
@@ -190,18 +221,58 @@ static int lastofs2 = 0;
 static int searchmode = 0;
 static char *searchstr = NULL;
 #endif
+#ifndef	_NOIME
+static int imemode = 0;
+#endif
+#ifndef	_NOKANJICONV
+static u_char ungetbuf3[MAXUTF8LEN * MAXNFLEN];
+static int ungetnum3 = 0;
+#endif
 
 
-int intrkey(VOID_A)
+#ifndef	_NOKANJICONV
+static int NEAR getinkcode(VOID_A)
 {
-	int c;
+	int code;
 
-	if (kbhit2(0L) && ((c = getkey2(0)) == cc_intr || c == K_ESC)) {
-		warning(0, INTR_K);
-		return(1);
+	code = (inputkcode != NOCNV) ? inputkcode : DEFCODE;
+# ifndef	_NOPTY
+	if (parentfd >= 0 && ptyinkcode != NOCNV) code = ptyinkcode;
+# endif
+
+	return(code);
+}
+#endif	/* !_NOKANJICONV */
+
+int intrkey(key)
+int key;
+{
+	int c, duperrno;
+
+	duperrno = errno;
+	if (ttyio < 0 || !kbhit2(0L)) c = EOF;
+	else if ((c = getch2()) == EOF) /*EMPTY*/;
+	else if (c != K_ESC) /*EMPTY*/;
+	else if (kbhit2(WAITKEYPAD * 1000L)) {
+		ungetch2(c);
+		c = EOF;
 	}
 
-	return(0);
+	if (c == EOF) /*EMPTY*/;
+	else if (c == cc_intr || (key >= 0 && c == key)) {
+		if (isttyiomode) warning(0, INTR_K);
+		else {
+			fprintf2(stderr, INTR_K);
+			fputnl(stderr);
+		}
+	}
+	else {
+		ungetch2(c);
+		c = EOF;
+	}
+	errno = duperrno;
+
+	return((c == EOF) ? 0 : 1);
 }
 
 #ifndef	_NOEDITMODE
@@ -220,6 +291,100 @@ CONST char table[];
 }
 #endif	/* !_NOEDITMODE */
 
+#ifndef	_NOIME
+static int NEAR getimebuf(buf, ptrp)
+CONST char *buf;
+int *ptrp;
+{
+	int c;
+
+	if (getinkcode() != EUC || !isekana(buf, *ptrp))
+		c = (u_char)(buf[*ptrp]);
+	else c = mkekana((u_char)(buf[++(*ptrp)]));
+	(*ptrp)++;
+
+	return(c);
+}
+
+static int NEAR getime(sig, chp, nowait)
+int sig, *chp, nowait;
+{
+	static char buf[MAXKANJIBUF + 1] = "";
+	static int next = 0;
+	static int lastline = 0;
+	int x, y, line;
+
+	if (!ime_cont) next = lastline = 0;
+	else if (next > 0) {
+		if (next >= sizeof(buf) || !(*chp = getimebuf(buf, &next)))
+			next = 0;
+		else return(0);
+	}
+
+	if (nowait) return(-1);
+
+	line = ptr2line(inputlen) + 1;
+	if (line < lastline) line = lastline;
+	ime_locate = locate3;
+	ime_line = lastline = line;
+	x = win_x - xpos;
+	y = win_y - ypos;
+# ifndef	_NOORIGSHELL
+	if (shellmode) {
+		ime_xposp = &win_x;
+		x += xpos;
+		y += ypos;
+	}
+# endif
+	locate2(0, line);
+	setcursor(inputlen, -1);
+	Xputterm(L_CLEAR);
+# ifndef	_NOORIGSHELL
+	if (shellmode) /*EMPTY*/;
+	else
+# endif	/* !_NOORIGSHELL */
+	ime_line += ypos;
+	locate2(x, y);
+
+	*chp = ime_inputkanji(sig, buf);
+	if (*chp < 0 || !ime_cont) {
+		locate2(0, line);
+		Xputterm(L_CLEAR);
+		locate2(x, y);
+		Xtflush();
+	}
+	if (*chp != K_ESC) {
+		if (*chp < 0) {
+			next = lastline = 0;
+			ime_inputkanji(0, NULL);
+		}
+		else if (!*chp) *chp = getimebuf(buf, &next);
+		return(0);
+	}
+
+	imemode = lastline = 0;
+	return(-1);
+}
+#endif	/* !_NOIME */
+
+#ifndef	_NOKANJICONV
+static int NEAR getkey4(sig)
+int sig;
+{
+	int n;
+
+	for (n = ungetnum3 - 1; n >= 0; n--) ungetch2((int)ungetbuf3[n]);
+# ifndef	_NOIME
+	if (imemode && !ungetnum3 && getime(sig, &n, 0) >= 0) /*EMPTY*/;
+	else
+# endif
+	n = getkey3(sig, getinkcode());
+
+	ungetnum3 = 0;
+	return(n);
+}
+#endif	/* !_NOKANJICONV */
+
 int Xgetkey(sig, eof)
 int sig, eof;
 {
@@ -229,13 +394,17 @@ int sig, eof;
 	if (sig < 0) {
 #ifndef	_NOEDITMODE
 		vistat = eof;
+#endif
+#ifndef	_NOIME
+		imemode = 0;
+		ime_inputkanji(0, NULL);
 #endif
 		prev = -1;
 		return('\0');
 	}
 
 	sig = sigalrm(sig);
-	ch = getkey3(sig);
+	ch = getkey4(sig);
 	if (eof && (ch != cc_eof || prev == ch)) eof = 0;
 	prev = ch;
 	if (eof) return(-1);
@@ -322,7 +491,7 @@ int sig, eof;
 				vistat |= VI_NEXT;
 				break;
 		}
-	} while ((vistat & VI_NEXT) && (ch = getkey3(sig)) > 0);
+	} while ((vistat & VI_NEXT) && (ch = getkey4(sig)) > 0);
 	else if (!strcmp(editmode, "wordstar"))
 		ch = getemulatekey(ch, wordstarkey);
 #endif	/* !_NOEDITMODE */
@@ -331,7 +500,8 @@ int sig, eof;
 }
 
 static int NEAR trquoteone(sp, s, cxp, len)
-char **sp, *s;
+char **sp;
+CONST char *s;
 int *cxp, len;
 {
 	int vw;
@@ -361,7 +531,7 @@ int *cxp, len;
 }
 
 static char *NEAR trquote(s, len, widthp)
-char *s;
+CONST char *s;
 int len, *widthp;
 {
 	char *cp, *buf;
@@ -377,7 +547,7 @@ int len, *widthp;
 }
 
 static int NEAR vlen(s, cx)
-char *s;
+CONST char *s;
 int cx;
 {
 	int v, r, rw, vw;
@@ -411,7 +581,7 @@ int cx;
 }
 
 static int NEAR rlen(s, cx2)
-char *s;
+CONST char *s;
 int cx2;
 {
 	int v, r, rw, vw;
@@ -445,7 +615,7 @@ int cx2;
 }
 
 static int NEAR vonkanji1(s, cx2)
-char *s;
+CONST char *s;
 int cx2;
 {
 	int v, r, rw, vw;
@@ -480,7 +650,7 @@ int cx2;
 }
 
 int kanjiputs2(s, len, ptr)
-char *s;
+CONST char *s;
 int len, ptr;
 {
 	char *buf;
@@ -502,7 +672,7 @@ int len, ptr;
 
 #if	FD >= 2
 static VOID NEAR kanjiputs3(s, cx2, len2, ptr, top)
-char *s;
+CONST char *s;
 int cx2, len2, ptr, top;
 {
 	int n, width;
@@ -537,7 +707,7 @@ int n;
 
 VOID cputstr(n, s)
 int n;
-char *s;
+CONST char *s;
 {
 	if (!s) cputspace(n);
 	else Xcprintf2("%-*.*k", n, n, s);
@@ -622,7 +792,7 @@ int x;
 }
 
 static VOID NEAR dumbputs(s, cx2, len2, max, ptr)
-char *s;
+CONST char *s;
 int cx2, len2, max, ptr;
 {
 	len2 -= ptr;
@@ -725,6 +895,18 @@ int x, y;
 	Xlocate(win_x, win_y);
 }
 
+#ifndef	_NOIME
+static VOID locate3(x, y)
+int x, y;
+{
+# ifndef	_NOORIGSHELL
+	if (shellmode) locate2(x, y);
+	else
+# endif
+	Xlocate(win_x = x, win_y = y);
+}
+#endif	/* !_NOIME */
+
 static VOID NEAR setcursor(cx, cx2)
 int cx, cx2;
 {
@@ -1432,7 +1614,7 @@ int *cxp, *cxp2, ch, *qtopp, *qp, *qedp;
 			if (cxp != &rptr && rptr > *qedp) rptr--;
 			if (cxp2 != &vptr && vptr > cx2) vptr--;
 			inputlen--;
-			*qedp = 0;
+			*qedp = -1;
 			return(0);
 		}
 		*qtopp = *cxp;
@@ -1450,7 +1632,7 @@ int *cxp, *cxp2, ch, *qtopp, *qp, *qedp;
 }
 
 static int NEAR insertstr(strins, ins, qtop, qp, qedp)
-char *strins;
+CONST char *strins;
 int ins, qtop, *qp, *qedp;
 {
 	int i, n, ch, pc, max;
@@ -1532,7 +1714,7 @@ int ins, qtop, *qp, *qedp;
 #ifndef	_NOCOMPLETE
 static VOID NEAR selectfile(argc, argv)
 int argc;
-char **argv;
+char *CONST *argv;
 {
 	char *cp;
 	int i, len, maxlen, duprow, dupminfilename, dupcolumns, dupdispmode;
@@ -1542,7 +1724,7 @@ char **argv;
 	dupcolumns = curcolumns;
 	dupdispmode = dispmode;
 	minfilename = n_column;
-	dispmode = 2;
+	dispmode = F_FILETYPE;
 
 	if (argv) {
 		selectlist = (namelist *)malloc2(argc * sizeof(namelist));
@@ -1581,7 +1763,8 @@ char **argv;
 				Xcputs2("wish to see them all? (y or n)");
 				Xtflush();
 				for (;;) {
-					if ((i = getkey2(0)) < K_MIN) {
+					i = getkey3(0, getinkcode());
+					if (i < K_MIN) {
 						i = toupper2(i);
 						if (i == 'Y' || i == 'N')
 							break;
@@ -1722,13 +1905,16 @@ int comline, cont, h;
 		}
 # endif
 		else if (pc != PC_NORMAL) /*EMPTY*/;
+# ifdef	_USEDOSPATH
+		else if (i == top && _dospath(&(inputbuf[i]))) i++;
+# endif
 		else if (inputbuf[i] == ':' || inputbuf[i] == '='
 		|| strchr(CMDLINE_DELIM, inputbuf[i]))
 			top = i + 1;
 	}
 	if (comline && top > 0) {
 		for (i = top - 1; i >= 0; i--)
-			if (inputbuf[i] != ' ' && inputbuf[i] != '\t') break;
+			if (!isblank2(inputbuf[i])) break;
 		if (i >= 0 && !strchr(SHELL_OPERAND, inputbuf[i])) comline = 0;
 	}
 # ifndef	_NOORIGSHELL
@@ -1764,7 +1950,7 @@ int comline, cont, h;
 		i = (s_isdir(&(selectlist[tmpfilepos]))) ? 1 : 0;
 		argv[0] = (char *)malloc2(n + i + 1);
 		memcpy(argv[0], selectlist[tmpfilepos].name, n);
-		if (i) argv[0][n] = '/';
+		if (i) argv[0][n] = _SC_;
 		argv[0][n + i] = '\0';
 		argc = 1;
 	}
@@ -1953,8 +2139,55 @@ int comline, cont, h;
 }
 #endif	/* !_NOCOMPLETE */
 
+#ifndef	_NOKANJICONV
+static u_int NEAR getucs2(ch)
+int ch;
+{
+	char buf[MAXUTF8LEN + 1];
+	int n;
+
+	n = 0;
+	if (ch & 0xff00) return((u_int)-1);
+	buf[n++] = ch;
+	if (!ismsb(ch)) /*EMPTY*/;
+	else if ((ch = getch3()) == EOF) return((u_int)-1);
+	else {
+		buf[n++] = ch;
+		if (isutf2(buf[0], buf[1])) /*EMPTY*/;
+		else if ((ch = getch3()) == EOF) {
+			ungetch3(buf[1]);
+			return((u_int)-1);
+		}
+		else buf[n++] = ch;
+	}
+	buf[n] = '\0';
+
+	return(ucs2fromutf8((u_char *)buf, NULL));
+}
+
+static VOID NEAR ungetch3(c)
+int c;
+{
+	if (ungetnum3 >= arraysize(ungetbuf3)) return;
+	memmove((char *)&(ungetbuf3[1]), (char *)&(ungetbuf3[0]),
+		ungetnum3 * sizeof(u_char));
+	ungetbuf3[0] = c;
+	ungetnum3++;
+}
+#endif	/* !_NOKANJICONV */
+
 static int NEAR getch3(VOID_A)
 {
+#ifndef	_NOIME
+	int c;
+#endif
+
+#ifndef	_NOKANJICONV
+	if (ungetnum3 > 0) return((int)ungetbuf3[--ungetnum3]);
+#endif
+#ifndef	_NOIME
+	if (imemode) return((getime(0, &c, 1) >= 0) ? c : EOF);
+#endif
 	if (!kbhit2(WAITMETA * 1000L)) return(EOF);
 	return(getch2());
 }
@@ -1964,8 +2197,10 @@ char *buf;
 int ch;
 {
 #ifndef	_NOKANJICONV
-	char tmpkanji[3 + 1];
-	int n, code;
+	u_short ubuf[MAXNFLEN];
+	char tmp[MAXUTF8LEN + 1];
+	u_int u;
+	int i, n, len, code;
 #endif
 	int ch2;
 
@@ -1985,54 +2220,48 @@ int ch;
 	}
 # endif
 #else	/* !_NOKANJICONV */
-	code = (inputkcode != NOCNV) ? inputkcode : DEFCODE;
-# ifndef	_NOPTY
-	if (parentfd >= 0 && ptyinkcode != NOCNV) code = ptyinkcode;
-# endif
-
+	code = getinkcode();
 	if (code == EUC && isekana2(ch)) {
-		tmpkanji[0] = (char)C_EKANA;
-		tmpkanji[1] = (ch & 0xff);
-		tmpkanji[2] = '\0';
-		kanjiconv(buf, tmpkanji, 2, code, DEFCODE, L_INPUT);
+		tmp[0] = (char)C_EKANA;
+		tmp[1] = (ch & 0xff);
+		tmp[2] = '\0';
+		kanjiconv(buf, tmp, MAXKLEN, code, DEFCODE, L_INPUT);
 		return(1);
 	}
 	if (code == SJIS && iskana2(ch)) {
-		tmpkanji[0] = ch;
-		tmpkanji[1] = '\0';
-		kanjiconv(buf, tmpkanji, 2, code, DEFCODE, L_INPUT);
+		tmp[0] = ch;
+		tmp[1] = '\0';
+		kanjiconv(buf, tmp, MAXKLEN, code, DEFCODE, L_INPUT);
 		return(1);
 	}
-	if (code == UTF8 || code == M_UTF8) {
-		if ((ch & 0xff00) || !ismsb(ch)) /*EMPTY*/;
-		else if ((ch2 = getch3()) == EOF) {
-			buf[0] = '\0';
-			return(-1);
+	if (code >= UTF8) {
+		i = 0;
+		for (;;) {
+			if ((u = getucs2(ch)) == (u_int)-1) {
+				if (i) ungetch3(ch);
+				buf[0] = '\0';
+				return(-1);
+			}
+			ubuf[i++] = u;
+			if (i >= MAXNFLEN) break;
+			if (code != M_UTF8 || (ch = getch3()) == EOF) break;
 		}
-		else if (isutf2(ch, ch2)) {
-			tmpkanji[0] = ch;
-			tmpkanji[1] = ch2;
-			tmpkanji[2] = '\0';
-			n = kanjiconv(buf, tmpkanji, 2,
-				code, DEFCODE, L_INPUT);
-			return(n);
+		len = i;
+		i = 0;
+		if (!len) u = ch;
+		else if (code != M_UTF8) u = ubuf[i++];
+		else u = ucs2denormalization(ubuf, &i, code - UTF8);
+
+		while (i < len) {
+			tmp[ucs2toutf8(tmp, 0, ubuf[i++])] = '\0';
+			for (n = 0; tmp[n]; n++) ungetch3(tmp[n]);
 		}
-		else if ((n = getch3()) == EOF) {
-			buf[0] = '\0';
-			return(-1);
-		}
-		else {
-			tmpkanji[0] = ch;
-			tmpkanji[1] = ch2;
-			tmpkanji[2] = n;
-			tmpkanji[3] = '\0';
-			n = kanjiconv(buf, tmpkanji, 3,
-				code, DEFCODE, L_INPUT);
+		tmp[ucs2toutf8(tmp, 0, u)] = '\0';
+		n = kanjiconv(buf, tmp, MAXKLEN, code, DEFCODE, L_INPUT);
 # ifdef	CODEEUC
-			if (isekana(buf, 0)) n = 1;
+		if (isekana(buf, 0)) n = 1;
 # endif
-			return(n);
-		}
+		return(n);
 	}
 #endif	/* !_NOKANJICONV */
 
@@ -2046,10 +2275,10 @@ int ch;
 		buf[1] = ch2;
 		buf[2] = '\0';
 #else
-		tmpkanji[0] = ch;
-		tmpkanji[1] = ch2;
-		tmpkanji[2] = '\0';
-		kanjiconv(buf, tmpkanji, 2, code, DEFCODE, L_INPUT);
+		tmp[0] = ch;
+		tmp[1] = ch2;
+		tmp[2] = '\0';
+		kanjiconv(buf, tmp, MAXKLEN, code, DEFCODE, L_INPUT);
 #endif
 		return(2);
 	}
@@ -2066,7 +2295,7 @@ int ch;
 }
 
 static VOID NEAR copyhist(hist, keep)
-char *hist;
+CONST char *hist;
 int keep;
 {
 #ifndef	_NOORIGSHELL
@@ -2423,7 +2652,7 @@ char **tmp;
 #endif	/* FD >= 2 */
 
 static VOID NEAR _inputstr_input(buf, vw)
-char *buf;
+CONST char *buf;
 int vw;
 {
 	int rw;
@@ -2455,7 +2684,7 @@ int def, comline, h;
 #if	FD >= 2
 	ALLOC_T searchsize;
 #endif
-	char *tmphist, buf[2 + 1];
+	char *tmphist, buf[MAXKLEN + 1];
 	int i, n, ch, ch2, ovptr, hist, quote, sig;
 
 	subwindow = 1;
@@ -2496,7 +2725,7 @@ int def, comline, h;
 		ch2 = ch;
 		ovptr = vptr;
 		if (quote) {
-			i = ch = getkey2(sigalrm(sig));
+			i = ch = getkey2(sigalrm(sig), getinkcode());
 			quote = 0;
 #if	MSDOS
 			switch (i) {
@@ -2607,6 +2836,19 @@ int def, comline, h;
 		}
 #endif	/* FD >= 2 */
 
+#ifndef	_NOIME
+		if (!imemode && !selectlist
+		&& imekey >= 0 && ch == imekey && ch != K_ESC) {
+				keyflush();
+# ifndef	_NOORIGSHELL
+				if (dumbmode) ringbell();
+				else
+# endif
+				imemode = 1;
+				continue;
+		}
+#endif	/* !_NOIME */
+
 		switch (ch) {
 			case K_RIGHT:
 				keyflush();
@@ -2796,10 +3038,9 @@ int def, comline, h;
 				keyflush();
 #ifndef	_NOCOMPLETE
 				if (!completable(h) || !selectlist) break;
-				i = completestr(0, -1, h);
-				if (i < 0) break;
-				if (iseol(vptr)) ovptr = -1;
 				ch = '\0';
+				i = completestr(0, -1, h);
+				if (i >= 0 && iseol(vptr)) ovptr = -1;
 #endif	/* !_NOCOMPLETE */
 				break;
 			case K_ESC:
@@ -2874,10 +3115,10 @@ int def, comline, h;
 }
 
 static VOID NEAR dispprompt(s, set)
-char *s;
+CONST char *s;
 int set;
 {
-	static char *prompt = NULL;
+	static CONST char *prompt = NULL;
 	char *buf;
 
 	if (set > 0) prompt = s;
@@ -2950,9 +3191,9 @@ int set;
 }
 
 char *inputstr(prompt, delsp, ptr, def, h)
-char *prompt;
+CONST char *prompt;
 int delsp, ptr;
-char *def;
+CONST char *def;
 int h;
 {
 	int i, len, ch, pc, qtop, quote, comline, dupwin_x, dupwin_y;
@@ -3124,15 +3365,15 @@ int h;
 	return(inputbuf);
 }
 
-static char *NEAR truncstr(s)
-char *s;
+static VOID NEAR truncstr(s)
+CONST char *s;
 {
 	int len;
 	char *cp, *tmp;
 
 	if ((len = strlen2(s) + YESNOSIZE - n_lastcolumn) <= 0
 	|| !(cp = strchr2(s, '[')) || !(tmp = strchr2(cp, ']')))
-		return(s);
+		return;
 
 	cp++;
 	len = tmp - cp - len;
@@ -3142,12 +3383,10 @@ char *s;
 	else if (isekana(cp, len - 1)) len--;
 #endif
 	strcpy(&(cp[len]), tmp);
-
-	return(s);
 }
 
 static int NEAR yesnomes(mes)
-char *mes;
+CONST char *mes;
 {
 	int len;
 
@@ -3307,7 +3546,7 @@ va_dcl
 
 VOID warning(no, s)
 int no;
-char *s;
+CONST char *s;
 {
 	char *tmp, *err;
 	int y, len, wastty, dupwin_x, dupwin_y;
@@ -3343,7 +3582,7 @@ char *s;
 	if (!(wastty = isttyiomode)) Xttyiomode(1);
 	keyflush();
 	do {
-		getkey2(sigalrm(1));
+		getkey3(sigalrm(1), getinkcode());
 	} while (kbhit2(WAITAFTERWARN * 1000L));
 	if (!wastty) Xstdiomode();
 
@@ -3365,7 +3604,7 @@ char *s;
 
 static int NEAR selectcnt(max, str, multi)
 int max;
-char **str;
+char *CONST *str;
 int multi;
 {
 	int i, len;
@@ -3380,10 +3619,11 @@ int multi;
 
 static int NEAR selectadj(max, x, str, tmpstr, xx, multi)
 int max, x;
-char **str, **tmpstr;
-int xx[], multi;
+CONST char *CONST *str;
+char **tmpstr;
+int *xx, multi;
 {
-	char *cp;
+	char *cp, **new;
 	int i, len, maxlen;
 
 	for (i = 0; i < max; i++) {
@@ -3396,29 +3636,29 @@ int xx[], multi;
 	else if ((x = n_lastcolumn - 1 - len) >= 0) /*EMPTY*/;
 	else {
 		x = maxlen = 0;
-		str = (char **)malloc2(max * sizeof(char *));
+		new = (char **)malloc2(max * sizeof(char *));
 		for (i = 0; i < max; i++) {
 			if (!(cp = tmpstr[i])) {
-				str[i] = NULL;
+				new[i] = NULL;
 				continue;
 			}
 			if (isupper2(*cp) && cp[1] == ':')
 				for (cp += 2; *cp == ' '; cp++);
 			len = strlen3(cp);
 			if (len > maxlen) maxlen = len;
-			str[i] = strdup2(cp);
+			new[i] = strdup2(cp);
 		}
 
 		for (; maxlen > 0; maxlen--) {
-			for (i = 0; i < max; i++) if (str[i]) {
+			for (i = 0; i < max; i++) if (new[i]) {
 				len = maxlen;
-				strncpy3(tmpstr[i], str[i], &len, -1);
+				strncpy3(tmpstr[i], new[i], &len, -1);
 			}
 			if (x + selectcnt(max, tmpstr, multi) < n_lastcolumn)
 				break;
 		}
-		for (i = 0; i < max; i++) if (str[i]) free(str[i]);
-		free(str);
+		for (i = 0; i < max; i++) if (new[i]) free(new[i]);
+		free(new);
 		if (maxlen <= 0) return(-1);
 	}
 
@@ -3437,8 +3677,8 @@ int xx[], multi;
 
 static VOID NEAR selectmes(num, max, x, str, val, xx, multi)
 int num, max, x;
-char *str[];
-int val[], xx[], multi;
+char *CONST str[];
+int val[], *xx, multi;
 {
 	int i;
 
@@ -3459,7 +3699,7 @@ int val[], xx[], multi;
 
 int selectstr(num, max, x, str, val)
 int *num, max, x;
-char *str[];
+CONST char *CONST str[];
 int val[];
 {
 	char **tmpstr;
