@@ -23,9 +23,12 @@
 #define	BSPATHDELIM
 #define	USECRNL
 #define	CWDINPATH
+#define	DOUBLESLASH
 #define	NOTZFILEH
 #define	USETIMEH
 #define	USEUTIME
+#define	NOFLOCK
+#define	NOSYSLOG
 #define	USEMKTIME
 #define	USESTRERROR
 #define	SENSEPERSEC	20
@@ -366,6 +369,7 @@ typedef long	off_t;
 #define	USEMNTCTL
 #define	USERE_COMP
 # if	defined (_AIX41)
+# define	HAVELONGLONG
 # define	USESTATVFSH
 # define	USEMKTIME
 # define	SIGFNCINT
@@ -470,6 +474,7 @@ typedef long	off_t;
 #define	PATHNOCASE
 #define	COMMNOCASE
 #define	USECRNL
+#define	DOUBLESLASH
 #define	USEMANLANG
 #define	BSDINSTALL
 #define	TARUSESPACE
@@ -487,6 +492,7 @@ typedef long	off_t;
 #define	USEMKTIME
 #define	DEFFDSETSIZE
 #define	SIGFNCINT
+#define	WAITKEYPAD		36
 #endif
 
 #if	defined (linux)
@@ -593,8 +599,6 @@ typedef long	off_t;
 #define	DECLSIGLIST
 #define	DECLERRLIST
 #define	USELEAPCNT
-#define	USEMOUNTH
-#define	USEMNTINFO
 #define	USEFFSTYPE
 #define	USERE_COMP
 #define	USESETENV
@@ -604,6 +608,13 @@ typedef long	off_t;
 # if	defined (NetBSD1_0) && (NetBSD1_0 < 1)
 # define	USEFFSIZE
 # endif
+# if	defined (__NetBSD_Version__) && (__NetBSD_Version__ >= 300000000)
+# define	USESTATVFSH
+# define	USEGETVFSTAT
+# else
+# define	USEMOUNTH
+# define	USEMNTINFO
+# endif
 #endif
 
 #if	defined (__bsdi__)
@@ -780,6 +791,7 @@ typedef long	off_t;
 /* #define BSPATHDELIM	;path delimtor is backspace  */
 /* #define USECRNL	;use CR-NL as end of line */
 /* #define CWDINPATH	;CWD is implicitly included in command path */
+/* #define DOUBLESLASH	;pathname starting with // has some special mean */
 /* #define USEMANLANG	;man(1) directory includes LANG environment value */
 /* #define SUPPORTSJIS	;cc(1) or man(1) supports Shift JIS perfectly */
 /* #define BSDINSTALL	;install(1) with option -c is valid like BSD */
@@ -827,10 +839,12 @@ typedef long	off_t;
 /* #define USESTDARGH	;use <stdarg.h> for va_list */
 /* #define USEMKDEVH	;use <sys/mkdev.h> for major()/minor() */
 /* #define USEMKNODH	;use <sys/mknod.h> for major()/minor() */
+/* #define USELOCKFH	;use <sys/lockf.h> for lockf() */
 /* #define USESGTTY	;use sgtty interface */
 /* #define USETERMIO	;use termio interface */
 /* #define USETERMIOS	;use termios interface */
 /* #define HAVECLINE	;struct termios has c_line */
+/* #define USEDEVPTMX	;use /dev/ptmx and /dev/pts/? for pseudo terminal */
 /* #define USEDIRECT	;use 'struct direct' instead of dirent */
 /* #define SYSVDIRENT	;dirent interface behaves as System V */
 /* #define NODNAMLEN	;struct dirent hasn't d_namlen */
@@ -854,6 +868,7 @@ typedef long	off_t;
 /* #define USEMNTENTH	;use <mntent.h> as header of the mount entry */
 /* #define USEMNTTABH	;use <sys/mnttab.h> as header of the mount entry */
 /* #define USEGETFSSTAT	;use getfsstat() to get the mount entry */
+/* #define USEGETVFSTAT	;use getvfsstat() to get the mount entry */
 /* #define USEMNTCTL	;use mntctl() to get the mount entry */
 /* #define USEMNTINFOR	;use getmntinfo_r() to get the mount entry */
 /* #define USEMNTINFO	;use getmntinfo() to get the mount entry */
@@ -878,6 +893,10 @@ typedef long	off_t;
 /* #define NOTERMVAR	;have not termcap variables such as PC, ospeed, etc. */
 /* #define USEUTIME	;use utime() instead of utimes() */
 /* #define USEGETWD	;use getwd() instead of getcwd() */
+/* #define USEFCNTLOCK	;use fcntl() lock instead of flock() */
+/* #define USELOCKF	;use lockf() instead of flock() */
+/* #define NOFLOCK	;have neither flock() nor lockf() */
+/* #define NOSYSLOG	;have not syslog() */
 /* #define USETIMELOCAL	;have timelocal() as inverse of localtime() */
 /* #define USEMKTIME	;use mktime() instead of timelocal() */
 /* #define USESYSCONF	;use sysconf() for getting system configuration */
@@ -965,6 +984,14 @@ typedef long	off_t;
 # endif
 #endif
 
+#ifndef	BSD4
+#define	USEDEVPTMX
+#endif
+
+#if	defined (POSIX) || defined (SYSV)
+#define	USEFCNTLOCK
+#endif
+
 /*                                 */
 /* Eval configurations by Configur */
 /*                                 */
@@ -1019,7 +1046,7 @@ typedef long	off_t;
 #define	ALLOC_T		size_t
 #define	VOID_A		void
 #else
-#define	ALLOC_T		unsigned
+#define	ALLOC_T		u_int
 #define	VOID_A
 #endif
 
@@ -1113,41 +1140,51 @@ typedef long	off_t;
 #endif
 
 #if	defined (USEMNTTABH) || defined (USEGETFSSTAT)
+# ifdef	USEGETVFSTAT
+# undef	USEGETVFSTAT
+# endif
+#endif
+
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT)
 # ifdef	USEMNTCTL
 # undef	USEMNTCTL
 # endif
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL)
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL)
 # ifdef	USEMNTINFOR
 # undef	USEMNTINFOR
 # endif
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR)
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR)
 # ifdef	USEMNTINFO
 # undef	USEMNTINFO
 # endif
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO)
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO)
 # ifdef	USEGETMNT
 # undef	USEGETMNT
 # endif
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO) || defined (USEGETMNT)
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO) || defined (USEGETMNT)
 # ifdef	USEGETFSENT
 # undef	USEGETFSENT
 # endif
 #endif
 
-#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) || defined (USEMNTCTL) \
-|| defined (USEMNTINFOR) || defined (USEMNTINFO) || defined (USEGETMNT) \
-|| defined (USEGETFSENT)
+#if	defined (USEMNTTABH) || defined (USEGETFSSTAT) \
+|| defined (USEGETVFSTAT) || defined (USEMNTCTL) || defined (USEMNTINFOR) \
+|| defined (USEMNTINFO) || defined (USEGETMNT) || defined (USEGETFSENT)
 # ifdef	USEMNTENTH
 # undef	USEMNTENTH
 # endif
@@ -1164,4 +1201,4 @@ typedef long	off_t;
 #define	GETTODARGS	2
 #endif
 
-#endif	/* __MACHINE_H_ */
+#endif	/* !__MACHINE_H_ */
