@@ -14,7 +14,7 @@
 # ifndef	DJGPP
 # include <dos.h>
 #  ifdef	__TURBOC__
-extern unsigned _stklen = 0x5800;
+extern u_int _stklen = 0x5800;
 #  define	harderr_t	void
 #  else
 #  define	harderr_t	int
@@ -32,6 +32,14 @@ extern unsigned _stklen = 0x5800;
 #define	isorgpid()	(mypid == orgpid)
 #endif
 
+#if	!defined (_NOIME) && defined (DEBUG)
+#include "roman.h"
+#endif
+
+#ifndef	DATADIR
+#define	DATADIR		progpath
+#endif
+
 #if	MSDOS
 # ifndef	BSPATHDELIM
 extern char *adjustpname __P_((char *));
@@ -58,7 +66,7 @@ extern int maxarchive;
 #endif
 extern int wheader;
 extern char fullpath[];
-extern int savehist;
+extern short savehist[];
 extern char *helpindex[];
 extern int subwindow;
 extern int win_x;
@@ -67,9 +75,12 @@ extern int win_y;
 extern int custno;
 #endif
 extern char *deftmpdir;
-#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+#ifdef	_USEUNICODE
 extern char *unitblpath;
 #endif
+#ifndef	_NOIME
+extern char *dicttblpath;
+#endif
 #ifndef	_NOPTY
 extern int ptymode;
 extern int ptyinternal;
@@ -147,19 +158,23 @@ static VOID NEAR printtime __P_((int));
 #ifdef	SIGALRM
 static int trapalrm __P_((VOID_A));
 #endif
+#ifndef	_NODOSDRIVE
+static int wrap_intrkey __P_((VOID_A));
+#endif
 static char *NEAR getversion __P_((int *));
 #ifndef	_NOLOGGING
-static VOID NEAR startlog __P_((char **));
+static VOID NEAR startlog __P_((char *CONST *));
 static VOID NEAR endlog __P_((int));
 #endif
 #ifdef	_NOORIGSHELL
-static int NEAR execruncomline __P_((char *, char *, int, char *));
+static int NEAR execruncomline __P_((char *, CONST char *, int, CONST char *));
 #endif
-static int NEAR initoption __P_((int, char *[], char *[]));
-static int NEAR evaloption __P_((char *[]));
-static char *NEAR searchenv __P_((char *, char *[]));
-static VOID NEAR setexecname __P_((char *));
-static VOID NEAR setexecpath __P_((char *, char *[]));
+static int NEAR initoption __P_((int, char *CONST [], char *CONST []));
+static int NEAR evaloption __P_((char *CONST []));
+static char *NEAR searchenv __P_((CONST char *, char *CONST []));
+static VOID NEAR setexecname __P_((CONST char *));
+static VOID NEAR setexecpath __P_((CONST char *, char *CONST []));
+int main __P_((int, char *CONST [], char *CONST []));
 
 char *origpath = NULL;
 char *progpath = NULL;
@@ -216,7 +231,7 @@ u_short far *devhdr;
 #endif
 
 VOID error(s)
-char *s;
+CONST char *s;
 {
 	int duperrno;
 
@@ -239,6 +254,7 @@ char *s;
 		fputs(s, stderr);
 		fputnl(stderr);
 	}
+	fclose(stderr);
 
 	if (isorgpid()) {
 		inittty(1);
@@ -495,13 +511,13 @@ VOID setlinecol(VOID_A)
 {
 	char buf[MAXLONGWIDTH + 1];
 
-	if (getconstvar("LINES")) {
+	if (getconstvar(ENVLINES)) {
 		snprintf2(buf, sizeof(buf), "%d", n_line);
-		setenv2("LINES", buf, 1);
+		setenv2(ENVLINES, buf, 1);
 	}
-	if (getconstvar("COLUMNS")) {
+	if (getconstvar(ENVCOLUMNS)) {
 		snprintf2(buf, sizeof(buf), "%d", n_column);
-		setenv2("COLUMNS", buf, 1);
+		setenv2(ENVCOLUMNS, buf, 1);
 	}
 }
 
@@ -541,7 +557,8 @@ int n, max;
 VOID checkscreen(xmax, ymax)
 int xmax, ymax;
 {
-	char *cp, *tty;
+	CONST char *cp;
+	char *tty;
 	int i, row, wastty, dupn_line, dupdumbterm;
 
 #ifdef	SIGWINCH
@@ -592,7 +609,7 @@ int xmax, ymax;
 		Xputterm(T_BELL);
 		Xcputnl();
 		Xtflush();
-		if (kbhit2(1000000L) && getkey2(0) == K_ESC) {
+		if (kbhit2(1000000L) && getkey3(0, inputkcode) == K_ESC) {
 			errno = 0;
 			error(INTR_K);
 		}
@@ -724,6 +741,13 @@ static int trapalrm(VOID_A)
 }
 #endif	/* SIGALRM */
 
+#ifndef	_NODOSDRIVE
+static int wrap_intrkey(VOID_A)
+{
+	return(intrkey(-1));
+}
+#endif	/* !_NODOSDRIVE */
+
 int sigvecset(set)
 int set;
 {
@@ -745,7 +769,7 @@ int set;
 #endif
 #ifndef	_NODOSDRIVE
 		doswaitfunc = waitmes;
-		dosintrfunc = intrkey;
+		dosintrfunc = wrap_intrkey;
 #endif
 		status = 1;
 	}
@@ -787,7 +811,7 @@ int *lenp;
 
 VOID title(VOID_A)
 {
-	char *cp;
+	CONST char *cp;
 	int i, len;
 
 	Xlocate(0, L_TITLE);
@@ -811,7 +835,7 @@ VOID title(VOID_A)
 		Xputch2('#');
 		i++;
 	}
-	cp = (iswellomit()) ? nullstr : " (c)1995-2006 T.Shirai  ";
+	cp = (iswellomit()) ? nullstr : " (c)1995-2007 T.Shirai  ";
 	Xcputs2(cp);
 	i = n_column - len - strlen2(cp) - i;
 	while (i-- > 0) Xputch2(' ');
@@ -842,7 +866,7 @@ VOID saveorigenviron(VOID_A)
 
 #ifndef	_NOLOGGING
 static VOID NEAR startlog(argv)
-char **argv;
+char *CONST *argv;
 {
 # ifndef	NOUID
 	uid_t uid;
@@ -897,9 +921,10 @@ int status;
 
 #ifdef	_NOORIGSHELL
 static int NEAR execruncomline(command, file, n, line)
-char *command, *file;
+char *command;
+CONST char *file;
 int n;
-char *line;
+CONST char *line;
 {
 	char *cp;
 	int i;
@@ -922,7 +947,7 @@ char *line;
 
 /*ARGSUSED*/
 int loadruncom(file, exist)
-char *file;
+CONST char *file;
 int exist;
 {
 #ifdef	_NOORIGSHELL
@@ -935,7 +960,7 @@ int exist;
 #ifdef	_NOORIGSHELL
 # if	!MSDOS
 	tmp = NULL;
-	if (!exist && (tmp = getconstvar("TERM"))) {
+	if (!exist && (tmp = getconstvar(ENVTERM))) {
 		struct stat st;
 
 		cp = malloc2(strlen(file) + strlen(tmp) + 1 + 1);
@@ -971,7 +996,7 @@ int exist;
 		}
 
 		cp = line + strlen(line);
-		for (cp--; cp >= line && (*cp == ' ' || *cp == '\t'); cp--);
+		for (cp--; cp >= line && isblank2(*cp); cp--);
 		cp[1] = '\0';
 
 		cont = 0;
@@ -1013,7 +1038,7 @@ int exist;
 /*ARGSUSED*/
 static int NEAR initoption(argc, argv, envp)
 int argc;
-char *argv[], *envp[];
+char *CONST argv[], *CONST envp[];
 {
 	char *cp, **optv;
 	int i, optc;
@@ -1050,7 +1075,7 @@ char *argv[], *envp[];
 }
 
 static int NEAR evaloption(argv)
-char *argv[];
+char *CONST argv[];
 {
 	char *cp, *tmp;
 	int i;
@@ -1066,13 +1091,13 @@ char *argv[];
 		setenv2(tmp, cp, 0);
 		free(tmp);
 	}
-	evalenv();
 
 	return(i);
 }
 
 static char *NEAR searchenv(s, envp)
-char *s, *envp[];
+CONST char *s;
+char *CONST envp[];
 {
 	int i, len;
 
@@ -1085,7 +1110,7 @@ char *s, *envp[];
 }
 
 static VOID NEAR setexecname(argv)
-char *argv;
+CONST char *argv;
 {
 #if	MSDOS || defined (CYGWIN)
 	char *cp;
@@ -1101,13 +1126,15 @@ char *argv;
 }
 
 static VOID NEAR setexecpath(argv, envp)
-char *argv, *envp[];
+CONST char *argv;
+char *CONST envp[];
 {
-	char *cp, buf[MAXPATHLEN];
+	CONST char *cp;
+	char *tmp, buf[MAXPATHLEN];
 
 	if (!Xgetwd(buf)) error(NOCWD_K);
 	origpath = strdup2(buf);
-	if ((cp = searchenv("PWD", envp))) {
+	if ((cp = searchenv(ENVPWD, envp))) {
 		*fullpath = '\0';
 		realpath2(cp, fullpath, 0);
 		realpath2(fullpath, buf, 1);
@@ -1118,26 +1145,30 @@ char *argv, *envp[];
 	}
 	strcpy(fullpath, origpath);
 
+	tmp = NULL;
 #if	MSDOS
 	cp = argv;
 #else
 	if (strdelim(argv, 0)) cp = argv;
-	else if ((cp = searchenv("PATH", envp))) cp = searchexecpath(argv, cp);
+	else if ((cp = searchenv(ENVPATH, envp)))
+		cp = tmp = searchexecpath(argv, cp);
 	if (!cp) progpath = strdup2(origpath);
 	else
 #endif
 	{
 		realpath2(cp, buf, 1);
-		if (cp != argv) free(cp);
-		if ((cp = strrdelim(buf, 0))) *cp = '\0';
+		if (tmp) free(tmp);
+		if ((tmp = strrdelim(buf, 0))) *tmp = '\0';
 		progpath = strdup2(buf);
 	}
 }
 
 /*ARGSUSED*/
 VOID initfd(argv)
-char **argv;
+char *CONST *argv;
 {
+	int i;
+
 #ifndef	_NOORIGSHELL
 	if (!interactive) /*EMPTY*/;
 	else
@@ -1150,7 +1181,7 @@ char **argv;
 			Xttyiomode(0);
 		}
 #endif	/* !MSDOS */
-		loadhistory(0);
+		for (i = 0; i < 2; i++) loadhistory(i);
 		entryhist(1, origpath, 1);
 	}
 #ifndef	_NOLOGGING
@@ -1163,6 +1194,7 @@ VOID prepareexitfd(status)
 int status;
 {
 	char cwd[MAXPATHLEN];
+	int i;
 
 #ifndef	_NOLOGGING
 	endlog(status);
@@ -1171,7 +1203,7 @@ int status;
 	if (!interactive) /*EMPTY*/;
 	else
 #endif
-	savehistory(0);
+	for (i = 0; i < 2; i++) savehistory(i);
 
 	cwd[0] = '\0';
 	if (origpath && _chdir2(origpath) < 0) {
@@ -1181,9 +1213,12 @@ int status;
 	}
 	free(origpath);
 	free(progname);
-#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+#ifdef	_USEUNICODE
 	free(unitblpath);
 #endif
+#ifndef	_NOIME
+	free(dicttblpath);
+#endif
 #ifndef	_NODOSDRIVE
 	dosallclose();
 #endif
@@ -1235,16 +1270,21 @@ int status;
 # ifndef	_NOROCKRIDGE
 	detranspath(NULL, NULL);
 # endif
-# if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
+# ifdef	_USEUNICODE
 	discardunitable();
 # endif
+# ifndef	_NOIME
+	ime_freebuf();
+	freeroman(0);
+	discarddicttable();
+# endif
 #endif	/* DEBUG */
 	if (*cwd) rawchdir(cwd);
 }
 
 int main(argc, argv, envp)
 int argc;
-char *argv[], *envp[];
+char *CONST argv[], *CONST envp[];
 {
 	char *cp;
 	int i;
@@ -1358,12 +1398,11 @@ char *argv[], *envp[];
 #endif
 
 	setexecpath(argv[0], envp);
-#if	!defined (_NOKANJICONV) || !defined (_NODOSDRIVE)
-# ifdef	DATADIR
+#ifdef	_USEUNICODE
 	unitblpath = strdup2(DATADIR);
-# else
-	unitblpath = strdup2(progpath);
-# endif
+#endif
+#ifndef	_NOIME
+	dicttblpath = strdup2(DATADIR);
 #endif
 
 #ifdef	_NOORIGSHELL
@@ -1392,6 +1431,7 @@ char *argv[], *envp[];
 		error(NTERM_K);
 	}
 #endif	/* !_NOORIGSHELL */
+	initenv();
 
 	Xttyiomode(0);
 	initterm();
