@@ -19,7 +19,7 @@ typedef struct _printbuf_t {
 	char *buf;
 	int ptr;
 	int size;
-	int flags;
+	u_short flags;
 } printbuf_t;
 
 #define	VF_NEW		000001
@@ -34,7 +34,7 @@ typedef struct _printbuf_t {
 #define	VF_UNSIGNED	001000
 
 #define	strsize(s)	((int)sizeof(s) - 1)
-#define	arraysize(a)	((int)sizeof(a) / (int)sizeof(*(a)))
+#define	arraysize(a)	((int)((u_int)sizeof(a) / (u_int)sizeof(*(a))))
 
 #define	MAXLONGWIDTH	20		/* log10(2^64) = 19.266 */
 #define	MAXCOLSCOMMA(d)	(MAXLONGWIDTH + (MAXLONGWIDTH / (d)))
@@ -61,10 +61,10 @@ typedef long			long_t;
 typedef unsigned long		u_long_t;
 #endif
 
-extern char printfflagchar[];
-extern int printfflag[];
-extern char printfsizechar[];
-extern int printfsize[];
+extern CONST char printfflagchar[];
+extern CONST int printfflag[];
+extern CONST char printfsizechar[];
+extern CONST int printfsize[];
 
 extern int getnum __P_((CONST char *, int *));
 extern int setchar __P_((int, printbuf_t *));
@@ -75,7 +75,7 @@ extern int snprintf2 __P_((char *, int, CONST char *, ...));
 extern int fprintf2 __P_((FILE *, CONST char *, ...));
 extern VOID fputnl __P_((FILE *));
 #ifdef	FD
-extern VOID kanjifputs __P_((char *, FILE *));
+extern VOID kanjifputs __P_((CONST char *, FILE *));
 #else
 #define	kanjifputs	fputs
 #endif
