@@ -4,18 +4,33 @@
  *	shell command module
  */
 
+#include <fcntl.h>
 #include "fd.h"
 #include "func.h"
 #include "kanji.h"
 
 #if	MSDOS && !defined (_NOUSELFN)
-extern char *shortname __P_((char *, char *));
+extern char *shortname __P_((CONST char *, char *));
 #endif
 
 #ifndef	_NOORIGSHELL
 #include "system.h"
 #endif
 
+#ifndef	_NOPTY
+#include "termemu.h"
+#endif
+
+#ifndef	O_TEXT
+#define	O_TEXT		0
+#endif
+
+typedef struct _localetable {
+	CONST char *env;
+	CONST char *val;
+	char *org;
+} localetable;
+
 extern int mark;
 extern off_t marksize;
 extern char fullpath[];
@@ -41,30 +56,31 @@ extern int internal_status;
 # endif
 #endif
 
-static int NEAR checksc __P_((char *, int, char *));
+static int NEAR checksc __P_((CONST char *, int, CONST char *));
 #ifndef	_NOKANJICONV
 static int NEAR extconv __P_((char **, int, int, ALLOC_T *, int));
 #endif
-static int NEAR isneedargs __P_((char *, int, int *));
-static int NEAR setarg __P_((char **, int, ALLOC_T *, char *, char *, int));
+static int NEAR isneedargs __P_((CONST char *, int, int *));
+static int NEAR setarg __P_((char **, int, ALLOC_T *,
+		CONST char *, CONST char *, int));
 static int NEAR flag2str __P_((char *, int, int));
-static int NEAR skipquote __P_((char *, int *));
+static int NEAR skipquote __P_((CONST char *, int *));
 #if	!defined (MACROMETA) || !defined (_NOEXTRAMACRO)
-static char *NEAR _restorearg __P_((char *));
+static char *NEAR _restorearg __P_((CONST char *));
 static char *NEAR _demacroarg __P_((char *));
 #endif
 #ifdef	_NOEXTRAMACRO
 static int NEAR setargs __P_((char **, int, ALLOC_T *,
-	int, int, int, macrostat *, int));
-static char *NEAR insertarg __P_((char *, char *, int));
+		int, int, int, macrostat *, int));
+static char *NEAR insertarg __P_((CONST char *, CONST char *, int));
 #else
-static int NEAR _replaceargs __P_((int *, char ***, char **, int));
+static int NEAR _replaceargs __P_((int *, char ***, char *CONST *, int));
 #endif
-static char *NEAR addoption __P_((char *, char *, macrostat *));
+static char *NEAR addoption __P_((char *, CONST char *, macrostat *));
 #ifdef	_NOORIGSHELL
-static int NEAR system3 __P_((char *, int));
-static char *NEAR evalargs __P_((char *, int, char *[]));
-static char *NEAR evalalias __P_((char *));
+static int NEAR system3 __P_((CONST char *, int));
+static char *NEAR evalargs __P_((CONST char *, int, char *CONST *));
+static char *NEAR evalalias __P_((CONST char *));
 #else
 #define	system3			system2
 #endif
@@ -75,23 +91,34 @@ int maxalias = 0;
 userfunctable userfunclist[MAXFUNCTABLE];
 int maxuserfunc = 0;
 #else
-char *promptstr2 = NULL;
+CONST char *promptstr2 = NULL;
 #endif
-char *promptstr = NULL;
+CONST char *promptstr = NULL;
 char **history[2] = {NULL, NULL};
-char *histfile = NULL;
+char *histfile[2] = {NULL, NULL};
 short histsize[2] = {0, 0};
 short histno[2] = {0, 0};
-int savehist = 0;
+short savehist[2] = {0, 0};
 int n_args = 0;
 
 static short histbufsize[2] = {0, 0};
+static localetable localelist[] = {
+	{"LC_COLLATE", "C", NULL},
+	{"LC_CTYPE", "", NULL},
+	{"LC_MESSAGES", "C", NULL},
+	{"LC_MONETARY", "C", NULL},
+	{"LC_NUMERIC", "C", NULL},
+	{"LC_TIME", "C", NULL},
+	{"LC_ALL", NULL, NULL},
+	{"LANG", "C", NULL},
+};
+#define	LOCALELISTSIZ	arraysize(localelist)
 
 
 static int NEAR checksc(buf, ptr, arg)
-char *buf;
+CONST char *buf;
 int ptr;
-char *arg;
+CONST char *arg;
 {
 	if (!arg || !*arg) /*EMPTY*/;
 	else if (*arg == _SC_ || isdotdir(arg)) /*EMPTY*/;
@@ -144,7 +171,7 @@ int code;
 #endif	/* !_NOKANJICONV */
 
 static int NEAR isneedargs(s, n, flagsp)
-char *s;
+CONST char *s;
 int n, *flagsp;
 {
 	int i;
@@ -189,38 +216,39 @@ static int NEAR setarg(bufp, ptr, sizep, dir, arg, flags)
 char **bufp;
 int ptr;
 ALLOC_T *sizep;
-char *dir, *arg;
+CONST char *dir, *arg;
 int flags;
 {
-	char *cp, path[MAXPATHLEN], conv[MAXPATHLEN];
+	char *cp, *new, path[MAXPATHLEN], conv[MAXPATHLEN];
 	int len, optr;
 
 	if (!arg || !*arg) return(checksc(*bufp, ptr, NULL) - ptr);
-	if (!dir || !*dir) {
+	if (!dir) {
 #if	MSDOS && !defined (_NOUSELFN)
 		if ((flags & F_TOSFN) && shortname(arg, path) == path)
 			arg = path;
+		else
 #endif
+		arg = convput(conv, arg, 1, 0, NULL, NULL);
 	}
 	else {
 		strcatdelim2(path, dir, arg);
 		arg = path;
 #if	defined (BSPATHDELIM) && !defined (_NOARCHIVE)
-		if (flags & F_ISARCH) for (len = 0; arg[len]; len++) {
-			if (iskanji1(arg, len)) len++;
-			if (arg[len] == _SC_) arg[len] = '/';
+		if (flags & F_ISARCH) for (len = 0; path[len]; len++) {
+			if (iskanji1(path, len)) len++;
+			if (path[len] == _SC_) path[len] = '/';
 		}
+#endif
+#ifndef	_NOKANJIFCONV
+		arg = kanjiconv2(conv, arg,
+			MAXPATHLEN - 1, DEFCODE, fnamekcode, L_FNAME);
 #endif
 	}
 
-	if (arg != path) arg = convput(conv, arg, 1, 0, NULL, NULL);
-#ifndef	_NOKANJIFCONV
-	else arg = kanjiconv2(conv, arg, MAXPATHLEN - 1,
-		DEFCODE, fnamekcode, L_FNAME);
-#endif
 	optr = ptr;
 	ptr = checksc(*bufp, ptr, arg);
-	arg = killmeta(arg);
+	arg = new = killmeta(arg);
 
 	if ((flags & F_NOEXT) && (cp = strrchr(arg, '.')) && cp != arg)
 		len = cp - arg;
@@ -228,7 +256,7 @@ int flags;
 
 	*bufp = c_realloc(*bufp, ptr + len + 1, sizep);
 	strncpy(&((*bufp)[ptr]), arg, len);
-	free(arg);
+	free(new);
 
 	return(len + ptr - optr);
 }
@@ -253,7 +281,7 @@ int ptr, flags;
 }
 
 static int NEAR skipquote(s, ptrp)
-char *s;
+CONST char *s;
 int *ptrp;
 {
 	static int q = '\0';
@@ -262,7 +290,7 @@ int *ptrp;
 	if (!*ptrp) q = '\0';
 	pc = parsechar(&(s[*ptrp]), -1, '\0', 0, &q, NULL);
 	if (pc == PC_NORMAL) return(0);
-	if (pc == PC_WORD || pc == PC_META) {
+	if (pc == PC_WORD || pc == PC_ESCAPE) {
 		(*ptrp)++;
 		return(2);
 	}
@@ -272,7 +300,7 @@ int *ptrp;
 
 #if	!defined (MACROMETA) || !defined (_NOEXTRAMACRO)
 static char *NEAR _restorearg(s)
-char *s;
+CONST char *s;
 {
 	char *buf;
 	int i, j, m, flags;
@@ -303,7 +331,7 @@ char *s;
 	}
 	if (!flags) {
 		free(buf);
-		return(s);
+		return((char *)s);
 	}
 	buf[j] = '\0';
 
@@ -430,13 +458,15 @@ int flags;
 }
 
 static char *NEAR insertarg(format, arg, needmark)
-char *format, *arg;
+CONST char *format, *arg;
 int needmark;
 {
 # if	MSDOS && !defined (_NOUSELFN)
-	char *org, alias[MAXPATHLEN];
+	CONST char *org;
+	char alias[MAXPATHLEN];
 # endif
-	char *cp, *src, *ins, *buf, conv[MAXPATHLEN];
+	CONST char *src, *ins;
+	char *cp, *new, *buf, conv[MAXPATHLEN];
 	ALLOC_T size;
 	int i, j, m, len, flags;
 
@@ -466,7 +496,7 @@ int needmark;
 	{
 		buf = c_realloc(buf, j + 1, &size);
 		memcpy(buf, format, j);
-		src = format + j + m;
+		src = &(format[j + m]);
 		for (i = 0; i < needmark; i++) {
 			ins = arg;
 # if	MSDOS && !defined (_NOUSELFN)
@@ -480,7 +510,7 @@ int needmark;
 			}
 # endif
 			j = checksc(buf, j, ins);
-			ins = killmeta(ins);
+			ins = new = killmeta(ins);
 
 			if ((flags & F_NOEXT) && (cp = strrchr(ins, '.')))
 				len = cp - ins;
@@ -490,7 +520,7 @@ int needmark;
 # endif
 			buf = c_realloc(buf, j + len + 1, &size);
 			strncpy(&(buf[j]), ins, len);
-			free(ins);
+			free(new);
 			j += len;
 			for (len = 0; src[len]; len++) {
 # ifndef	MACROMETA
@@ -521,14 +551,14 @@ int needmark;
 #endif	/* _NOEXTRAMACRO */
 
 char *evalcommand(command, arg, stp)
-char *command, *arg;
+CONST char *command, *arg;
 macrostat *stp;
 {
 #ifndef	_NOKANJICONV
 	int code, cnvcode, defcode, cnvptr;
 #endif
 	macrostat st;
-	char *cp, *line;
+	char *cp, *line, *new, conv[MAXPATHLEN];
 	ALLOC_T size;
 	int i, j, c, pc, len, setflag, flags;
 
@@ -656,6 +686,7 @@ macrostat *stp;
 				else if (c == 'C') code = CAP;
 				else if (c == 'U') code = UTF8;
 				else if (c == 'M') code = M_UTF8;
+				else if (c == 'I') code = I_UTF8;
 #  ifndef	_NOKANJIFCONV
 				else if (c == 'A') code = -1;
 #  endif
@@ -745,8 +776,6 @@ macrostat *stp;
 #endif	/* _NOEXTRAMACRO */
 
 	if (!(flags & F_ARGSET) && arg && *arg) {
-		char conv[MAXPATHLEN];
-
 		arg = convput(conv, arg, 1, 1, NULL, NULL);
 		if (checksc(NULL, 0, arg) < 0) cp = NULL;
 		else {
@@ -756,7 +785,7 @@ macrostat *stp;
 			strcpy(&(cp[2]), arg);
 			arg = cp;
 		}
-		arg = killmeta(arg);
+		arg = new = killmeta(arg);
 		if (cp) free(cp);
 
 		len = strlen(arg);
@@ -770,7 +799,7 @@ macrostat *stp;
 			strncpy(&(line[j]), arg, len);
 			j += len;
 		}
-		free(arg);
+		free(new);
 	}
 	if (!(flags & F_IGNORELIST)
 	&& !(stp -> needburst) && !(stp -> needmark)) {
@@ -790,7 +819,7 @@ macrostat *stp;
 /*ARGSUSED*/
 int replaceargs(argcp, argvp, env, iscomm)
 int *argcp;
-char ***argvp, **env;
+char ***argvp, *CONST *env;
 int iscomm;
 {
 	return(0);
@@ -807,7 +836,7 @@ char **argp;
 
 static int NEAR _replaceargs(argcp, argvp, env, iscomm)
 int *argcp;
-char ***argvp, **env;
+char ***argvp, *CONST *env;
 int iscomm;
 {
 	static int lastptr = 0;
@@ -1066,7 +1095,7 @@ int iscomm;
 
 int replaceargs(argcp, argvp, env, iscomm)
 int *argcp;
-char ***argvp, **env;
+char ***argvp, *CONST *env;
 int iscomm;
 {
 # ifdef	MACROMETA
@@ -1101,12 +1130,12 @@ char **argp;
 #endif	/* !_NOEXTRAMACRO */
 
 char *restorearg(s)
-char *s;
+CONST char *s;
 {
 #if	defined (MACROMETA) && !defined (_NOEXTRAMACRO)
 	return(_restorearg(s));
 #else
-	return(s);
+	return((char *)s);
 #endif
 }
 
@@ -1120,16 +1149,17 @@ char **argp;
 }
 
 char *inputshellstr(prompt, ptr, def)
-char *prompt;
+CONST char *prompt;
 int ptr;
-char *def;
+CONST char *def;
 {
-	char *cp, *tmp, *duppromptstr;
+	CONST char *duppromptstr;
+	char *cp, *tmp;
 	int wastty;
 
 	duppromptstr = promptstr;
 	if (prompt) promptstr = prompt;
-	cp = inputstr(NULL, 0, ptr, def, 0);
+	cp = inputstr(NULL, 0, ptr, def, HST_COM);
 	promptstr = duppromptstr;
 	if (!cp) return((char *)-1);
 
@@ -1147,7 +1177,7 @@ char *def;
 
 char *inputshellloop(ptr, def)
 int ptr;
-char *def;
+CONST char *def;
 {
 #ifndef	_NOORIGSHELL
 	syntaxtree *trp, *stree;
@@ -1200,7 +1230,8 @@ char *def;
 }
 
 static char *NEAR addoption(command, arg, stp)
-char *command, *arg;
+char *command;
+CONST char *arg;
 macrostat *stp;
 {
 	char *cp;
@@ -1258,7 +1289,7 @@ macrostat *stp;
 #endif	/* MACROMETA */
 
 	if (!(wastty = isttyiomode)) Xttyiomode(1);
-	cp = inputstr("", 0, p, command, 0);
+	cp = inputstr(nullstr, 0, p, command, HST_COM);
 	if (!wastty) Xstdiomode();
 	free(command);
 	if (!cp) {
@@ -1282,13 +1313,13 @@ macrostat *stp;
 
 #ifdef	_NOORIGSHELL
 static int NEAR system3(command, flags)
-char *command;
+CONST char *command;
 int flags;
 {
 	char *cp, *tmp;
 	int n, ret;
 
-	while (*command == ' ' || *command == '\t') command++;
+	command = skipspace(command);
 	if ((cp = evalalias(command))) command = cp;
 
 	n = sigvecset(0);
@@ -1305,7 +1336,7 @@ int flags;
 #endif	/* _NOORIGSHELL */
 
 int isinternalcomm(command)
-char *command;
+CONST char *command;
 {
 #ifndef	_NOORIGSHELL
 	syntaxtree *stree;
@@ -1340,14 +1371,14 @@ char *command;
 }
 
 int execmacro(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
 #ifdef	_NOEXTRAMACRO
 	char *buf;
 	int r, status;
 #endif
-	static char *duparg = NULL;
+	static CONST char *duparg = NULL;
 	macrostat st;
 	char *tmp;
 	int i, haslist, ret;
@@ -1355,7 +1386,7 @@ int flags;
 	if (arg) duparg = arg;
 	haslist = (filelist && !(flags & F_IGNORELIST));
 	ret = 0;
-	internal_status = -2;
+	internal_status = FNC_FAIL;
 
 	if (!haslist || !mark) n_args = 0;
 	else {
@@ -1393,7 +1424,7 @@ int flags;
 		if (!(flags & F_ARGSET)) st.flags &= ~F_ARGSET;
 
 		if (r > ret && (ret = r) >= 127) break;
-		if (internal_status < -1) status = 4;
+		if (internal_status <= FNC_FAIL) status = FNC_EFFECT;
 		else if (internal_status > status) status = internal_status;
 
 		if (!(st.flags & F_REMAIN)
@@ -1422,7 +1453,7 @@ int flags;
 			r = system3(buf, st.flags);
 			free(buf);
 			if (r > ret && (ret = r) >= 127) break;
-			if (internal_status < -1) status = 4;
+			if (internal_status <= FNC_FAIL) status = FNC_EFFECT;
 			else if (internal_status > status)
 				status = internal_status;
 		}
@@ -1430,11 +1461,11 @@ int flags;
 	internal_status = status;
 #else	/* !_NOEXTRAMACRO */
 	ret = system3(tmp, st.flags);
-	if (internal_status < -1) internal_status = 4;
+	if (internal_status <= FNC_FAIL) internal_status = FNC_EFFECT;
 #endif	/* !_NOEXTRAMACRO */
 
 	if (tmp) free(tmp);
-	if (haslist && internal_status < -1) {
+	if (haslist && internal_status <= FNC_FAIL) {
 		for (i = 0; i < maxfile; i++)
 			filelist[i].tmpflags &= ~(F_ISARG | F_ISMRK);
 		mark = 0;
@@ -1446,23 +1477,34 @@ int flags;
 }
 
 FILE *popenmacro(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
 	macrostat st;
 	FILE *fp;
 	char *tmp, *lang;
+	int i;
 
-	internal_status = -2;
+	internal_status = FNC_FAIL;
 	st.flags = flags;
 	if (isinternalcomm(command)) st.flags |= F_ARGSET;
 
 	if (!(tmp = evalcommand(command, arg, &st))) return(NULL);
-	if ((lang = strdup2(getenv2("LANG")))) setenv2("LANG", "C", 1);
+	for (i = 0; i < LOCALELISTSIZ; i++) {
+		localelist[i].org = strdup2(getenv2(localelist[i].env));
+		if (!(localelist[i].val) || *(localelist[i].val)) {
+			setenv2(localelist[i].env, localelist[i].val, 1);
+			continue;
+		}
+		lang = getenv2("LC_ALL");
+		if (!lang || !*lang) lang = getenv2(localelist[i].env);
+		if (!lang || !*lang) lang = getenv2("LANG");
+		setenv2(localelist[i].env, lang, 1);
+	}
 	fp = popen2(tmp);
-	if (lang) {
-		setenv2("LANG", lang, 1);
-		free(lang);
+	for (i = 0; i < LOCALELISTSIZ; i++) {
+		setenv2(localelist[i].env, localelist[i].org, 1);
+		if (localelist[i].org) free(localelist[i].org);
 	}
 	free(tmp);
 
@@ -1471,9 +1513,9 @@ int flags;
 
 #ifdef	_NOORIGSHELL
 static char *NEAR evalargs(command, argc, argv)
-char *command;
+CONST char *command;
 int argc;
-char *argv[];
+char *CONST *argv;
 {
 	char *cp, *line;
 	ALLOC_T size;
@@ -1508,7 +1550,7 @@ char *argv[];
 }
 
 int execusercomm(command, arg, flags)
-char *command, *arg;
+CONST char *command, *arg;
 int flags;
 {
 	char *cp, **argv;
@@ -1519,13 +1561,13 @@ int flags;
 	if (i >= maxuserfunc) ret = execmacro(command, arg, flags);
 	else {
 		ret = 0;
-		status = internal_status = -2;
+		status = internal_status = FNC_FAIL;
 		for (j = 0; userfunclist[i].comm[j]; j++) {
 			cp = evalargs(userfunclist[i].comm[j], argc, argv);
 			r = execmacro(cp, arg, flags);
 			free(cp);
 			if (r > ret && (ret = r) >= 127) break;
-			if (internal_status < -1) status = 4;
+			if (internal_status <= FNC_FAIL) status = FNC_EFFECT;
 			else if (internal_status > status)
 				status = internal_status;
 		}
@@ -1537,7 +1579,7 @@ int flags;
 }
 
 static char *NEAR evalalias(command)
-char *command;
+CONST char *command;
 {
 	char *cp;
 	int i, len;
@@ -1555,28 +1597,29 @@ char *command;
 
 int entryhist(n, s, uniq)
 int n;
-char *s;
+CONST char *s;
 int uniq;
 {
+	char *new;
 	int i, size;
 
 	size = (int)histsize[n];
 	if (!history[n]) {
-		history[n] = (char **)malloc2(sizeof(char *) * (size + 1));
+		history[n] = (char **)malloc2((size + 1) * sizeof(char *));
 		for (i = 0; i <= size; i++) history[n][i] = NULL;
 		histbufsize[n] = size;
 		histno[n] = (short)0;
 	}
 	else if (size > histbufsize[n]) {
 		history[n] = (char **)realloc2(history[n],
-			sizeof(char *) * (size + 1));
+			(size + 1) * sizeof(char *));
 		for (i = histbufsize[n] + 1; i <= size; i++)
 			history[n][i] = NULL;
 		histbufsize[n] = size;
 	}
 
 	if (!s || !*s) return(0);
-	s = (n == 1) ? killmeta(s) : strdup2(s);
+	new = (n == 1) ? killmeta(s) : strdup2(s);
 
 	if (histno[n]++ >= MAXHISTNO) histno[n] = (short)0;
 
@@ -1584,16 +1627,19 @@ int uniq;
 		for (i = 0; i <= size; i++) {
 			if (!history[n][i]) continue;
 #if	defined (PATHNOCASE) && defined (_USEDOSPATH)
-			if (n == 1 && *s != *(history[n][i])) continue;
+			if (n == 1 && *new != *(history[n][i])) continue;
 #endif
-			if (!strpathcmp(s, history[n][i])) break;
+			if (!strpathcmp(new, history[n][i])) break;
 		}
 		if (i < size) size = i;
 	}
 
 	if (history[n][size]) free(history[n][size]);
 	for (i = size; i > 0; i--) history[n][i] = history[n][i - 1];
-	history[n][0] = s;
+	history[n][0] = new;
+#ifndef	_NOPTY
+	sendparent(TE_SETHISTORY, n, s, uniq);
+#endif
 
 	return(1);
 }
@@ -1614,30 +1660,34 @@ int n;
 	return(tmp);
 }
 
-int loadhistory(n, file)
+int loadhistory(n)
 int n;
-char *file;
 {
-	FILE *fp;
+	lockbuf_t *lck;
 	char *line;
 	int i, j, size;
 
-	if (!(fp = Xfopen(file, "r"))) return(-1);
+	if (!histfile[n] || !histfile[n][0]) return(0);
+	lck = lockfopen(histfile[n], "r", O_TEXT | O_RDONLY);
+	if (!lck || !(lck -> fp)) {
+		lockclose(lck);
+		return(-1);
+	}
 
 	size = (int)histsize[n];
-	history[n] = (char **)malloc2(sizeof(char *) * (size + 1));
+	history[n] = (char **)malloc2((size + 1) * sizeof(char *));
 	histbufsize[n] = size;
 	histno[n] = (short)0;
 
 	i = -1;
-	while ((line = fgets2(fp, 1))) {
+	while ((line = fgets2(lck -> fp, 1))) {
 		if (histno[n]++ >= MAXHISTNO) histno[n] = (short)0;
 		if (i < size) i++;
 		else free(history[n][i]);
 		for (j = i; j > 0; j--) history[n][j] = history[n][j - 1];
 		history[n][0] = line;
 	}
-	Xfclose(fp);
+	lockclose(lck);
 
 	for (i++; i <= size; i++) history[n][i] = NULL;
 
@@ -1645,7 +1695,7 @@ char *file;
 }
 
 VOID convhistory(s, fp)
-char *s;
+CONST char *s;
 FILE *fp;
 {
 	char *eol;
@@ -1661,29 +1711,35 @@ FILE *fp;
 	VOID_C Xfputc('\n', fp);
 }
 
-int savehistory(n, file)
+int savehistory(n)
 int n;
-char *file;
 {
-	FILE *fp;
+	lockbuf_t *lck;
 	int i, size;
 
+	if (!histfile[n] || !histfile[n][0] || savehist[n] <= 0) return(0);
 	if (!history[n] || !history[n][0]) return(-1);
-	if (!(fp = Xfopen(file, "w"))) return(-1);
+	lck = lockfopen(histfile[n], "w",
+		O_TEXT | O_WRONLY | O_CREAT | O_TRUNC);
+	if (!lck || !(lck -> fp)) {
+		lockclose(lck);
+		return(-1);
+	}
 
-	size = (savehist > (int)histsize[n]) ? (int)histsize[n] : savehist;
-	for (i = size - 1; i >= 0; i--) convhistory(history[n][i], fp);
-	Xfclose(fp);
+	size = (savehist[n] > histsize[n])
+		? (int)histsize[n] : (int)savehist[n];
+	for (i = size - 1; i >= 0; i--) convhistory(history[n][i], lck -> fp);
+	lockclose(lck);
 
 	return(0);
 }
 
-int parsehist(str, ptrp)
-char *str;
-int *ptrp;
+int parsehist(str, ptrp, quote)
+CONST char *str;
+int *ptrp, quote;
 {
 	char *cp;
-	int i, n, ptr, size;
+	int i, n, pc, ptr, size;
 
 	ptr = (ptrp) ? *ptrp : 0;
 	size = (int)histsize[0];
@@ -1701,8 +1757,10 @@ int *ptrp;
 	}
 	else {
 		for (i = 0; str[ptr + i]; i++) {
-			if (strchr(CMDLINE_DELIM, str[ptr + i])) break;
-			if (iskanji1(str, ptr + i)) i++;
+			pc = parsechar(&(str[ptr + i]), -1,
+				'!', EA_FINDDELIM, &quote, NULL);
+			if (pc == PC_WORD || pc == PC_ESCAPE) i++;
+			else if (pc != PC_NORMAL && pc != PC_DQUOTE) break;
 		}
 		if (!i) n = ptr = -1;
 		else {
@@ -1717,7 +1775,10 @@ int *ptrp;
 
 	if (ptrp) *ptrp = ptr;
 	if (n < 0 || n >= size) return(-1);
-	if (!ptrp) {
+	if (ptrp) {
+		if (!history[0][n]) return(-1);
+	}
+	else {
 		while (!history[0][n]) if (--n < 0) return(-1);
 	}
 
@@ -1736,10 +1797,10 @@ char *command;
 	for (i = j = 0, quote = '\0'; command[i]; i++) {
 		cp = c_realloc(cp, j + 1, &size);
 		pc = parsechar(&(command[i]), -1, '!', 0, &quote, NULL);
-		if (pc == PC_WORD || pc == PC_META) cp[j++] = command[i++];
-		else if (pc == '!'){
+		if (pc == PC_WORD || pc == PC_ESCAPE) cp[j++] = command[i++];
+		else if (pc == '!') {
 			len = i++;
-			if ((n = parsehist(command, &i)) < 0) {
+			if ((n = parsehist(command, &i, quote)) < 0) {
 				if (i < 0) {
 					cp[j++] = '!';
 					i = len;
@@ -1787,7 +1848,7 @@ int n;
 
 #if	!defined (_NOCOMPLETE) && defined (_NOORIGSHELL)
 int completealias(com, len, argc, argvp)
-char *com;
+CONST char *com;
 int len, argc;
 char ***argvp;
 {
@@ -1806,7 +1867,7 @@ char ***argvp;
 }
 
 int completeuserfunc(com, len, argc, argvp)
-char *com;
+CONST char *com;
 int len, argc;
 char ***argvp;
 {
