@@ -4,11 +4,11 @@
  *	terminal module
  */
 
+#include "machine.h"
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
 #include <fcntl.h>
-#include "machine.h"
 
 #if	defined (DJGPP) && (DJGPP >= 2)
 #include <time.h>
@@ -49,8 +49,8 @@ static int _asm_cli __P_((char *));
 #include <sys/time.h>
 
 typedef struct _kstree_t {
-	int key;
-	int num;
+	u_char key;
+	u_char num;
 	struct _kstree_t *next;
 } kstree_t;
 #endif	/* !MSDOS */
@@ -78,11 +78,17 @@ extern int errno;
 #define	strerror2		strerror
 #else
 # ifndef	DECLERRLIST
-extern char *sys_errlist[];
+extern CONST char *CONST sys_errlist[];
 # endif
 #define	strerror2(n)		(char *)sys_errlist[n]
 #endif
 
+#ifdef	_POSIX_VDISABLE
+#define	K_UNDEF		_POSIX_VDISABLE
+#else
+#define	K_UNDEF		0xff
+#endif
+
 #ifdef	DEBUG
 extern VOID muntrace __P_ ((VOID_A));
 extern char *_mtrace_file;
@@ -207,11 +213,11 @@ typedef int	tputs_t;
 # define	TERM_at1		key_beg
 # define	TERM_at7		key_end
 # else	/* !USETERMINFO */
-extern int tgetent __P_((char *, char *));
-extern int tgetnum __P_((char *));
-extern int tgetflag __P_((char *));
-extern char *tgetstr __P_((char *, char **));
-extern int tputs __P_((char *, int, int (*)__P_((tputs_t))));
+extern int tgetent __P_((char *, CONST char *));
+extern int tgetnum __P_((CONST char *));
+extern int tgetflag __P_((CONST char *));
+extern char *tgetstr __P_((CONST char *, char **));
+extern int tputs __P_((CONST char *, int, int (*)__P_((tputs_t))));
 # define	tgetnum2		tgetnum
 # define	tgetflag2		tgetflag
 # define	TERM_pc			"pc"
@@ -334,9 +340,9 @@ extern int tputs __P_((char *, int, int (*)__P_((tputs_t))));
 #endif
 #endif	/* !MSDOS */
 
-static int NEAR err2 __P_((char *));
+static int NEAR err2 __P_((CONST char *));
 #if	!MSDOS
-static char *NEAR tstrdup __P_((char *));
+static char *NEAR tstrdup __P_((CONST char *));
 #endif
 static int NEAR defaultterm __P_((VOID_A));
 static int NEAR maxlocate __P_((int *, int *));
@@ -347,7 +353,7 @@ static int NEAR biosscroll __P_((int, int, int, int, int));
 static int NEAR biosputch __P_((int, int));
 static int NEAR bioscurstype __P_((int));
 static int NEAR chgattr __P_((int));
-static int NEAR evalcsi __P_((char *));
+static int NEAR evalcsi __P_((CONST char *));
 # endif
 # if	!defined (DJGPP) || defined (NOTUSEBIOS) || defined (PC98)
 static int NEAR dosgettime __P_((u_char []));
@@ -358,9 +364,9 @@ static int NEAR ttymode __P_((int, int, int, int));
 # else
 static int NEAR ttymode __P_((int, int, int, int, int, int, int, int));
 # endif
-static char *NEAR tgetstr2 __P_((char **, char *));
-static char *NEAR tgetstr3 __P_((char **, char *, char *));
-static char *NEAR tgetkeyseq __P_((int, char *));
+static char *NEAR tgetstr2 __P_((char **, CONST char *));
+static char *NEAR tgetstr3 __P_((char **, CONST char *, CONST char *));
+static char *NEAR tgetkeyseq __P_((int, CONST char *));
 static kstree_t *NEAR newkeyseqtree __P_((kstree_t *, int));
 static int NEAR freekeyseqtree __P_((kstree_t *, int));
 static int cmpkeyseq __P_((CONST VOID_P, CONST VOID_P));
@@ -383,7 +389,11 @@ int n_column = -1;
 int n_lastcolumn = -1;
 int n_line = -1;
 int stable_standout = 0;
+#if	MSDOS
+CONST char *termstr[MAXTERMSTR];
+#else
 char *termstr[MAXTERMSTR];
+#endif
 u_char cc_intr = K_CTRL('C');
 u_char cc_quit = K_CTRL('\\');
 #if	MSDOS
@@ -391,7 +401,7 @@ u_char cc_eof = K_CTRL('Z');
 #else
 u_char cc_eof = K_CTRL('D');
 #endif
-u_char cc_eol = 255;
+u_char cc_eol = K_UNDEF;
 u_char cc_erase = K_CTRL('H');
 int (*keywaitfunc)__P_((VOID_A)) = NULL;
 #if	!MSDOS
@@ -437,7 +447,7 @@ static u_short keybuftop = 0;
 # ifdef	USEVIDEOBIOS
 static u_short videoattr = 0x07;
 # endif
-static int specialkeycode[] = {
+static CONST int specialkeycode[] = {
 	0,
 	K_UP, K_DOWN, K_RIGHT, K_LEFT,
 	K_IC, K_DC, K_HOME, K_END, K_PPAGE, K_NPAGE,
@@ -448,11 +458,11 @@ static int specialkeycode[] = {
 };
 #define	SPECIALKEYSIZ	arraysize(specialkeycode)
 #else	/* !MSDOS */
-static char *dumblist[] = {"dumb", "un", "unknown"};
+static CONST char *dumblist[] = {"dumb", "un", "unknown"};
 #define	DUMBLISTSIZE	arraysize(dumblist)
 static keyseq_t keyseq[K_MAX - K_MIN + 1];
 static kstree_t *keyseqtree = NULL;
-static char *defkeyseq[K_MAX - K_MIN + 1] = {
+static CONST char *defkeyseq[K_MAX - K_MIN + 1] = {
 	NULL,			/* K_NOKEY */
 	"\033OB",		/* K_DOWN */
 	"\033OA",		/* K_UP */
@@ -571,7 +581,7 @@ static char *defkeyseq[K_MAX - K_MIN + 1] = {
 #endif	/* !MSDOS */
 
 #if	MSDOS || !defined (TIOCSTI)
-static u_char ungetbuf[10];
+static u_char ungetbuf[16];
 static int ungetnum = 0;
 #endif
 
@@ -582,7 +592,7 @@ static int termflags = 0;
 #define	F_TTYCHANGED	010
 #define	F_INPROGRESS	020
 #define	F_RESETTTY	(F_INITTTY | F_TTYCHANGED)
-static char *deftermstr[MAXTERMSTR] = {
+static CONST char *deftermstr[MAXTERMSTR] = {
 #ifdef	PC98
 	"\033[>1h",		/* T_INIT */
 	"\033[>1l",		/* T_END */
@@ -869,7 +879,7 @@ int reset;
 		cc_quit = cc.t_quitc;
 		cc_eof = cc.t_eofc;
 		cc_eol = cc.t_brkc;
-		if (cc_erase != 255) cc_erase = dupttyio.sg_erase;
+		if (cc_erase != K_UNDEF) cc_erase = dupttyio.sg_erase;
 #else
 # ifdef	VINTR
 		cc_intr = dupttyio.c_cc[VINTR];
@@ -883,7 +893,7 @@ int reset;
 # ifdef	VEOL
 		cc_eol = dupttyio.c_cc[VEOL];
 # endif
-		if (cc_erase != 255) cc_erase = dupttyio.c_cc[VERASE];
+		if (cc_erase != K_UNDEF) cc_erase = dupttyio.c_cc[VERASE];
 #endif
 #ifndef	USETERMINFO
 		ospeed = getspeed(dupttyio);
@@ -1201,7 +1211,7 @@ int no;
 }
 
 static int NEAR err2(mes)
-char *mes;
+CONST char *mes;
 {
 	int duperrno;
 
@@ -1229,7 +1239,7 @@ char *mes;
 
 #if	!MSDOS
 static char *NEAR tstrdup(s)
-char *s;
+CONST char *s;
 {
 	char *cp;
 
@@ -1282,7 +1292,7 @@ static int NEAR defaultterm(VOID_A)
 #if	!MSDOS
 	for (i = 0; i <= K_MAX - K_MIN; i++) keyseq[i].code = K_MIN + i;
 	for (i = 21; i <= 30; i++)
-		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | 01000;
+		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | K_ALTERNATE;
 #endif	/* !MSDOS */
 
 	return(0);
@@ -1362,41 +1372,42 @@ int *xp, *yp;
 	tflush();
 # endif
 
-	i = 0;
-	buf[i] = '\0';
+	buf[0] = '\0';
 	do {
-		if (!kbhit2(WAITKEYPAD * 1000L * 2)) break;
+		if (!kbhit2(WAITKEYPAD * 1000L * 10)) break;
+# if	MSDOS
+		buf[0] = bdos(0x07, 0x00, 0);
+# else
+		if ((tmp = getch2()) == EOF) break;
+		buf[0] = tmp;
+# endif
+	} while (buf[0] != format[0]);
+
+	i = 0;
+	if (buf[0] == format[0]) for (i++; i < strsize(buf); i++) {
+		if (!kbhit2(WAITKEYPAD * 1000L)) break;
 # if	MSDOS
 		buf[i] = bdos(0x07, 0x00, 0);
 # else
 		if ((tmp = getch2()) == EOF) break;
 		buf[i] = tmp;
-# endif
-	} while (buf[i] != format[0]);
-
-	if (buf[i] == format[0]) while (i < strsize(buf) - 1) {
-		if (!kbhit2(WAITKEYPAD * 1000L)) break;
-# if	MSDOS
-		buf[++i] = bdos(0x07, 0x00, 0);
-# else
-		if ((tmp = getch2()) == EOF) break;
-		buf[++i] = tmp;
 # endif
 		if (buf[i] == format[strsize(SIZEFMT) - 1]) break;
 	}
 	keyflush();
-	if (!i || buf[i] != format[strsize(SIZEFMT) - 1]) return(-1);
-	buf[++i] = '\0';
+	while (kbhit2(WAITKEYPAD * 1000L)) VOID_C getch2();
+	if (!i || buf[i++] != format[strsize(SIZEFMT) - 1]) return(-1);
+	buf[i] = '\0';
 
 	count = 0;
 	val[0] = yp;
 	val[1] = xp;
 
 	for (i = j = 0; format[i] && buf[j]; i++) {
-		if (format[i] == '%' && format[++i] == 'd' && count < 2) {
-			tmp = getnum(buf, &j);
-			if (tmp < 0 || buf[j] != format[i + 1]) break;
-			*val[count++] = tmp;
+		if (format[i] == '%' && format[i + 1] == 'd') {
+			if ((tmp = getnum(buf, &j)) < 0) break;
+			i++;
+			if (count++ < 2) *(val[count - 1]) = tmp;
 		}
 		else if (format[i] != buf[j++]) break;
 	}
@@ -1408,7 +1419,7 @@ int *xp, *yp;
 
 #if	MSDOS
 char *tparamstr(s, arg1, arg2)
-char *s;
+CONST char *s;
 int arg1, arg2;
 {
 	char *cp;
@@ -1420,7 +1431,7 @@ int arg1, arg2;
 
 /*ARGSUSED*/
 int getterment(s)
-char *s;
+CONST char *s;
 {
 	if ((termflags & F_TERMENT) || (termflags & F_INPROGRESS)) return(-1);
 	termflags |= F_INPROGRESS;
@@ -1438,14 +1449,14 @@ char *s;
 #else	/* !MSDOS */
 
 char *tparamstr(s, arg1, arg2)
-char *s;
+CONST char *s;
 int arg1, arg2;
 {
 # ifdef	USETERMINFO
 #  ifdef	DEBUG
 	if (!s) return(NULL);
 	_mtrace_file = "tparm(start)";
-	s = tparm(s, arg1, arg2, 0, 0, 0, 0, 0, 0, 0);
+	s = tparm((char *)s, arg1, arg2, 0, 0, 0, 0, 0, 0, 0);
 	if (_mtrace_file) _mtrace_file = NULL;
 	else {
 		_mtrace_file = "tparm(end)";
@@ -1453,7 +1464,7 @@ int arg1, arg2;
 	}
 	if (!s) return(NULL);
 #  else
-	if (!s || !(s = tparm(s, arg1, arg2, 0, 0, 0, 0, 0, 0, 0)))
+	if (!s || !(s = tparm((char *)s, arg1, arg2, 0, 0, 0, 0, 0, 0, 0)))
 		return(NULL);
 #  endif
 
@@ -1559,7 +1570,7 @@ int arg1, arg2;
 
 static char *NEAR tgetstr2(term, s)
 char **term;
-char *s;
+CONST char *s;
 {
 # ifndef	USETERMINFO
 	char strbuf[TERMCAPSIZE];
@@ -1587,43 +1598,49 @@ char *s;
 }
 
 static char *NEAR tgetstr3(term, str1, str2)
-char **term, *str1, *str2;
+char **term;
+CONST char *str1, *str2;
 {
-# ifdef	USETERMINFO
-	if (str1) str1 = tstrdup(str1);
-	else str1 = tparamstr(str2, 1, 1);
-# else	/* !USETERMINFO */
+# ifndef	USETERMINFO
 	char *p, strbuf[TERMCAPSIZE];
+# endif
+	char *str;
 
+# ifdef	USETERMINFO
+	str = (str1) ? tstrdup(str1) : tparamstr(str2, 1, 1);
+# else	/* !USETERMINFO */
 	p = strbuf;
-
 #  ifdef	DEBUG
 	_mtrace_file = "tgetstr(start)";
-	str1 = tgetstr(str1, &p);
+	str = tgetstr(str1, &p);
 	if (_mtrace_file) _mtrace_file = NULL;
 	else {
 		_mtrace_file = "tgetstr(end)";
 		malloc(0);	/* dummy alloc */
 	}
-	if (str1) str1 = tstrdup(str1);
+	if (str) str = tstrdup(str);
 	else {
 		_mtrace_file = "tgetstr(start)";
-		str2 = tgetstr(str2, &p);
+		str = tgetstr(str2, &p);
 		if (_mtrace_file) _mtrace_file = NULL;
 		else {
 			_mtrace_file = "tgetstr(end)";
 			malloc(0);	/* dummy alloc */
 		}
-		str1 = tparamstr(str2, 1, 1);
+		str = tparamstr(str, 1, 1);
 	}
 #  else
-	if ((str1 = tgetstr(str1, &p))) str1 = tstrdup(str1);
-	else str1 = tparamstr(tgetstr(str2, &p), 1, 1);
+	str = tgetstr(str1, &p);
+	if (str) str = tstrdup(str);
+	else {
+		str = tgetstr(str2, &p);
+		str = tparamstr(str, 1, 1);
+	}
 #  endif
 # endif	/* !USETERMINFO */
-	if (str1) {
+	if (str) {
 		if (*term) free(*term);
-		*term = str1;
+		*term = str;
 	}
 
 	return(*term);
@@ -1631,7 +1648,7 @@ char **term, *str1, *str2;
 
 static char *NEAR tgetkeyseq(n, s)
 int n;
-char *s;
+CONST char *s;
 {
 	char *cp;
 	int i, j;
@@ -1673,13 +1690,13 @@ int num;
 	if (!parent) n = 0;
 	else {
 		n = parent -> num;
-		parent -> num = num;
+		parent -> num = (u_char)num;
 		parent -> next = new;
 	}
 
 	for (i = n; i < num; i++) {
-		new[i].key = -1;
-		new[i].num = 0;
+		new[i].key = (u_char)-1;
+		new[i].num = (u_char)0;
 		new[i].next = (kstree_t *)NULL;
 	}
 
@@ -1693,7 +1710,8 @@ int n;
 	int i;
 
 	if (!list) return(-1);
-	for (i = list[n].num - 1; i >= 0; i--) freekeyseqtree(list[n].next, i);
+	for (i = (int)(list[n].num) - 1; i >= 0; i--)
+		freekeyseqtree(list[n].next, i);
 	if (!n) free(list);
 
 	return(0);
@@ -1736,7 +1754,7 @@ static int NEAR sortkeyseq(VOID_A)
 			}
 			if (k >= p -> num) {
 				newkeyseqtree(p, k + 1);
-				p -> next[k].key = i;
+				p -> next[k].key = (u_char)i;
 			}
 			p = &(p -> next[k]);
 		}
@@ -1746,7 +1764,7 @@ static int NEAR sortkeyseq(VOID_A)
 }
 
 int getterment(s)
-char *s;
+CONST char *s;
 {
 # ifdef	IRIX
 	/* for STUPID winterm entry */
@@ -1755,7 +1773,8 @@ char *s;
 # ifndef	USETERMINFO
 	char buf[TERMCAPSIZE];
 # endif
-	char *cp, *term;
+	CONST char *term;
+	char *cp;
 	int i, j, dumb, dupdumbterm;
 
 	if ((termflags & F_TERMENT) || (termflags & F_INPROGRESS)) return(-1);
@@ -1763,7 +1782,7 @@ char *s;
 
 	dupdumbterm = dumbterm;
 	dumbterm = dumb = 0;
-	term = (s) ? s : (char *)getenv("TERM");
+	term = (s) ? s : (char *)getenv(ENVTERM);
 	if (!term || !*term) {
 		dumbterm = 1;
 		dumb = DUMBLISTSIZE;
@@ -1782,14 +1801,14 @@ char *s;
 # ifdef	USETERMINFO
 #  ifdef	DEBUG
 		_mtrace_file = "setupterm(start)";
-		setupterm(term, fileno(ttyout), &i);
+		setupterm((char *)term, fileno(ttyout), &i);
 		if (_mtrace_file) _mtrace_file = NULL;
 		else {
 			_mtrace_file = "setupterm(end)";
 			malloc(0);	/* dummy alloc */
 		}
 #  else
-		setupterm(term, fileno(ttyout), &i);
+		setupterm((char *)term, fileno(ttyout), &i);
 #  endif
 		if (i == 1) break;
 # else	/* !USETERMINFO */
@@ -1822,7 +1841,7 @@ char *s;
 	}
 
 	if (dupdumbterm >= 2 || !strcmp(term, "emacs")
-	|| ((cp = getenv("EMACS")) && !strcmp(cp, "t")))
+	|| ((cp = getenv(ENVEMACS)) && !strcmp(cp, "t")))
 		dumbterm = 3;
 
 	defaultterm();
@@ -2000,9 +2019,8 @@ char *s;
 			for (j = 0; keyseq[i].str[j]; j++)
 				keyseq[i].str[j] &= 0x7f;
 			keyseq[i].len = j;
-			if (cc_erase != 255 && j == 1
-			&& keyseq[i].str[0] == cc_erase)
-				cc_erase = 255;
+			if (j == 1 && keyseq[i].str[0] == cc_erase)
+				cc_erase = K_UNDEF;
 		}
 	}
 	sortkeyseq();
@@ -2061,7 +2079,7 @@ int setdefkeyseq(VOID_A)
 	}
 	for (i = 0; i <= K_MAX - K_MIN; i++) keyseq[i].code = K_MIN + i;
 	for (i = 21; i <= 30; i++)
-		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | 01000;
+		keyseq[K_F(i) - K_MIN].code = K_F(i - 20) | K_ALTERNATE;
 	sortkeyseq();
 
 	return(0);
@@ -2070,12 +2088,12 @@ int setdefkeyseq(VOID_A)
 int getdefkeyseq(kp)
 keyseq_t *kp;
 {
-	char *cp;
+	CONST char *cp;
 
 	if (kp -> code < K_MIN || kp -> code > K_MAX) /*EMPTY*/;
 	else if (!(cp = defkeyseq[kp -> code - K_MIN])) /*EMPTY*/;
 	else {
-		kp -> str = cp;
+		kp -> str = (char *)cp;
 		kp -> len = strlen(cp);
 		return(0);
 	}
@@ -2104,7 +2122,7 @@ int len;
 
 	if (str) {
 		for (i = 0; i <= K_MAX - K_MIN; i++) {
-			if ((keyseq[i].code & 0777) == n
+			if ((keyseq[i].code & ~K_ALTERNATE) == n
 			|| !(keyseq[i].str) || keyseq[i].len != len)
 				continue;
 			if (!memcmp(str, keyseq[i].str, len)) {
@@ -2113,8 +2131,7 @@ int len;
 				keyseq[i].len = 0;
 			}
 		}
-		if (cc_erase != 255 && str[0] == cc_erase && !(str[1]))
-			cc_erase = 255;
+		if (str[0] == cc_erase && !(str[1])) cc_erase = K_UNDEF;
 	}
 	sortkeyseq();
 
@@ -2153,7 +2170,8 @@ keyseq_t *list;
 				keyseq[i].str =
 					(char *)malloc(list[i].len + 1);
 				if (!keyseq[i].str) err2("malloc()");
-				memcpy(keyseq[i].str, list[i].str, list[i].len);
+				memcpy(keyseq[i].str, list[i].str,
+					list[i].len);
 			}
 		}
 		sortkeyseq();
@@ -2431,7 +2449,7 @@ int n;
 }
 
 static int NEAR evalcsi(s)
-char *s;
+CONST char *s;
 {
 	static int savex, savey;
 	int i, x, y, w, h, n1, n2;
@@ -2568,7 +2586,7 @@ char *s;
 }
 
 int cputs2(s)
-char *s;
+CONST char *s;
 {
 	int i, n;
 
@@ -2590,7 +2608,7 @@ int c;
 }
 
 int cputs2(s)
-char *s;
+CONST char *s;
 {
 	int i, x, y;
 
@@ -2747,8 +2765,8 @@ u_char tbuf[];
 #endif
 
 /*ARGSUSED*/
-int getkey2(sig)
-int sig;
+int getkey2(sig, code)
+int sig, code;
 {
 #if	!defined (DJGPP) || defined (NOTUSEBIOS) || defined (PC98)
 	static u_char tbuf1[3] = {0xff, 0xff, 0xff};
@@ -2815,10 +2833,9 @@ int sig;
 int ungetch2(c)
 int c;
 {
-	if (ungetnum >= arraysize(ungetbuf) - 1) return(EOF);
-	if (ungetnum)
-		memmove(&(ungetbuf[1]), &(ungetbuf[0]),
-			ungetnum * sizeof(u_char));
+	if (ungetnum >= arraysize(ungetbuf)) return(EOF);
+	memmove((char *)&(ungetbuf[1]), (char *)&(ungetbuf[0]),
+		ungetnum * sizeof(u_char));
 	ungetbuf[0] = c;
 	ungetnum++;
 
@@ -2883,7 +2900,7 @@ int c;
 }
 
 int cputs2(s)
-char *s;
+CONST char *s;
 {
 	if (!s) return(0);
 
@@ -2897,7 +2914,7 @@ tputs_t c;
 }
 
 int tputs2(s, n)
-char *s;
+CONST char *s;
 int n;
 {
 	return(tputs(s, n, putch3));
@@ -2945,8 +2962,9 @@ int getch2(VOID_A)
 	return((int)ch);
 }
 
-int getkey2(sig)
-int sig;
+/*ARGSUSED*/
+int getkey2(sig, code)
+int sig, code;
 {
 	static int count = SENSEPERSEC;
 	kstree_t *p;
@@ -2973,9 +2991,9 @@ int sig;
 	else if (key != C_EKANA) /*EMPTY*/;
 #  if	!defined (_NOKANJICONV)
 #   ifdef	CODEEUC
-	else if (inputkcode != EUC && inputkcode != NOCNV) /*EMPTY*/;
+	else if (code != EUC && code != NOCNV) /*EMPTY*/;
 #   else
-	else if (inputkcode != EUC) /*EMPTY*/;
+	else if (code != EUC) /*EMPTY*/;
 #   endif
 #  endif	/* !_NOKANJICONV */
 	else {
@@ -2987,7 +3005,7 @@ int sig;
 	}
 # endif	/* !_NOKANJICONV || CODEEUC */
 
-	if (cc_erase != 255 && key == cc_erase) return(K_BS);
+	if (cc_erase != K_UNDEF && key == cc_erase) return(K_BS);
 	if (!(p = keyseqtree)) return(key);
 
 	if (key == K_ESC) {
@@ -3009,8 +3027,7 @@ int sig;
 			if (key == keyseq[p -> next[j].key].str[0]) break;
 		if (j >= p -> num) return(key);
 		p = &(p -> next[j]);
-		if (keyseq[p -> key].len == 1)
-			return(keyseq[p -> key].code);
+		if (keyseq[p -> key].len == 1) return(keyseq[p -> key].code);
 		if (!kbhit2(WAITKEYPAD * 1000L) || (ch = getch2()) == EOF)
 			return(key);
 	}
@@ -3034,18 +3051,18 @@ int sig;
 	return(key);
 }
 
-int getkey3(sig)
-int sig;
+int getkey3(sig, code)
+int sig, code;
 {
 	int ch;
 
-	if ((ch = getkey2(sig)) < 0) return(ch);
+	if ((ch = getkey2(sig, code)) < 0) return(ch);
 	if (ch >= K_F('*') && ch < K_DL) {
 		if (ch == K_F('?')) ch = K_CR;
 		else ch -= K_F0;
 	}
 
-	return(ch & 01777);
+	return(ch & ~K_ALTERNATE);
 }
 
 int ungetch2(c)
@@ -3057,10 +3074,9 @@ int c;
 	ch = c;
 	Xioctl(ttyio, TIOCSTI, &ch);
 # else
-	if (ungetnum >= arraysize(ungetbuf) - 1) return(EOF);
-	if (ungetnum)
-		memmove(&(ungetbuf[1]), &(ungetbuf[0]),
-			ungetnum * sizeof(u_char));
+	if (ungetnum >= arraysize(ungetbuf)) return(EOF);
+	memmove((char *)&(ungetbuf[1]), (char *)&(ungetbuf[0]),
+		ungetnum * sizeof(u_char));
 	ungetbuf[0] = c;
 	ungetnum++;
 # endif
@@ -3233,7 +3249,7 @@ int cputnl(VOID_A)
 }
 
 int kanjiputs(s)
-char *s;
+CONST char *s;
 {
 	return(cprintf2("%k", s));
 }
