@@ -4,12 +4,10 @@
  *	terminal I/O
  */
 
+#include "machine.h"
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
-#include "machine.h"
-#include "printf.h"
-#include "termio.h"
 
 #ifdef	USESELECTH
 #include <sys/select.h>
@@ -23,6 +21,9 @@
 #include <stdlib.h>
 #endif
 
+#include "printf.h"
+#include "termio.h"
+
 #include <errno.h>
 #ifdef	NOERRNO
 extern int errno;
@@ -98,7 +99,7 @@ typedef struct fd_set {
 #define	Xwrite		write
 #else
 extern int Xread __P_((int, char *, int));
-extern int Xwrite __P_((int, char *, int));
+extern int Xwrite __P_((int, CONST char *, int));
 #endif
 
 #ifdef	LSI_C
@@ -302,7 +303,7 @@ int nbytes;
 
 int surewrite(fd, buf, nbytes)
 int fd;
-VOID_P buf;
+CONST VOID_P buf;
 int nbytes;
 {
 	char *cp;
@@ -362,7 +363,7 @@ FILE **fpp;
 #endif
 
 	if (*fdp >= 0) fd = *fdp;
-	else if ((fd = newdup(open(_PATH_TTY, flags, 0600))) < 0) return(-1);
+	else if ((fd = newdup(open(_PATH_TTY, flags, 0666))) < 0) return(-1);
 	if (*fpp) fp = *fpp;
 	else if (!(fp = fdopen(fd, "w+b")) && !(fp = fopen(_PATH_TTY, "w+b")))
 		return(-1);
@@ -394,7 +395,7 @@ FILE **fpp;
 /*ARGSUSED*/
 VOID loadtermio(fd, tty, ws)
 int fd;
-char *tty, *ws;
+CONST char *tty, *ws;
 {
 # ifndef	DJGPP
 	union REGS reg;
@@ -485,7 +486,7 @@ termioctl_t *t;
 
 int Xtcsetattr(fd, action, t)
 int fd, action;
-termioctl_t *t;
+CONST termioctl_t *t;
 {
 	for (;;) {
 		if (tcsetattr(fd, action, t) >= 0) {
@@ -528,7 +529,7 @@ int fd, selector;
 /*ARGSUSED*/
 VOID loadtermio(fd, tty, ws)
 int fd;
-char *tty, *ws;
+CONST char *tty, *ws;
 {
 	ALLOC_T size;
 
@@ -616,7 +617,8 @@ p_id_t Xfork(VOID_A)
 #endif	/* CYGWIN */
 
 #if	!MSDOS \
-|| (!defined(NOTUSEBIOS) && defined (DJGPP) && (DJGPP >= 2) && !defined(PC98))
+|| (!defined (NOTUSEBIOS) && !defined (PC98) \
+&& defined (DJGPP) && (DJGPP >= 2))
 int readselect(nfd, fds, result, vp)
 int nfd, fds[];
 char result[];
